from dataset_utils import minimal_metadata

from interp_latlon_plot import plot_latlon_interp_proj
from interp_latlon_plot import plot_latlon_interp

from tile_exchange import append_border_to_tile
from tile_exchange import add_borders_to_DataArray_V_points
from tile_exchange import add_borders_to_DataArray_U_points
from tile_exchange import add_borders_to_DataArray_G_points
from tile_exchange import get_llc_tile_border_mapping
from tile_exchange import add_borders_to_GRID_tiles

from tile_io import load_all_tiles_from_netcdf
from tile_io import load_tile_from_netcdf

from tile_plot import plot_tile
from tile_plot import plot_tiles
from tile_plot import plot_tiles_proj
from tile_plot import unique_color

from tile_rotation import reorient_13_tile_GRID_Dataset_to_latlon_layout
from tile_rotation import reorient_13_tile_Dataset_to_latlon_layout_CG_points
from tile_rotation import rotate_single_tile_Dataset_CG_points
from tile_rotation import rotate_single_tile_DataArray_CG_points
from tile_rotation import reorient_13_tile_Dataset_to_latlon_layout_UV_points
from tile_rotation import rotate_single_tile_Datasets_UV_points
from tile_rotation import rotate_single_tile_DataArrays_UV_points

from mds_io import load_llc_mds

__all__ = ['dataset_utils','io_utils', 'llc_plot','tile_exchange','tile_io','tile_rotation','interp_latlon_plot']



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""ECCO v4 Python: mds_io

This module includes utility routines for loading binary files in the llc 13-tile native flat binary layout.  This layout is the default for MITgcm input and output for global setups using lat-lon-cap (llc) layout.  The llc layout is used for ECCO v4. 

.. _ecco_v4_py Documentation :
   https://github.com/ECCO-GROUP/ECCOv4-py
"""

from __future__ import division
import numpy as np
import matplotlib.pylab as plt
import xarray as xr
import time
from copy import deepcopy
import glob

#%%
def load_llc_mds(fdir, fname, llc):
    """

    This routine loads a single 2D binary file in the llc layout

    Parameters
    ----------
    fdir : string
        A string with the directory of the binary file to open
    fname : string
        A string with the name of the binary file to open
    llc : int
        the size of the llc grid.  For ECCO v4, we use the llc90 domain so `llc` would be `90`
    
    Returns
    -------
    ndarray
        the binary file contents organized into a llc x llc x 13 `ndarray`, one llc x llc array for each of the 13

    Raises
    ------
    IOError
        If the file is not found

    """
    
    datafile = fdir + '/' + fname
    
    print 'loading ' + fname
    
    
    # check to see if file exists.    
    file = glob.glob(fname)
    if len(file) == 0:
        raise IOError(fname + ' not found ')


    f = open(datafile, 'rb')
    arr = np.fromfile(f, dtype='>f', count=-1)
    f.close()
    
    arr = np.reshape(arr,(llc*13, llc))
    
    f1 = arr[:3*llc,:]
    f2 = arr[3*llc:6*llc,:]
    f3 = arr[6*llc:7*llc,:]
    
    f4 = np.zeros((llc, 3*llc))

    for f in range(8,11):
        i1 = np.arange(0, llc)+(f-8)*llc
        i2 = np.arange(0,3*llc,3) + 7*llc + f -8
        f4[:,i1] = arr[i2,:]

    f5 = np.zeros((llc, 3*llc))

    for f in range(11,14):
        i1 = np.arange(0, llc)+(f-11)*llc
        i2 = np.arange(0,3*llc,3) + 10*llc + f -11
        f5[:,i1] = arr[i2,:]
    
    if 1 == 0:
        plt.close('all')
        plt.imshow(f1, origin='lower')
        plt.figure()
        plt.imshow(f2, origin='lower')   
        plt.figure()
        plt.imshow(f3, origin='lower')
        
        plt.figure()
        plt.imshow(f4, origin='lower')
        
        plt.figure()
        plt.imshow(f5, origin='lower')
    
    arr_tiles = np.zeros((13, 90, 90))
    arr_tiles[0,:] = f1[llc*0:llc*1,:]
    arr_tiles[1,:] = f1[llc*1:llc*2,:]
    arr_tiles[2,:] = f1[llc*2:,:]

    arr_tiles[3,:] = f2[llc*0:llc*1,:]
    arr_tiles[4,:] = f2[llc*1:llc*2,:]
    arr_tiles[5,:] = f2[llc*2:,:]
    
    arr_tiles[6,:] = f3

    arr_tiles[7,:] = f4[:,llc*0:llc*1]
    arr_tiles[8,:] = f4[:,llc*1:llc*2]
    arr_tiles[9,:] = f4[:,llc*2:]
    
    arr_tiles[10,:] = f5[:,llc*0:llc*1]
    arr_tiles[11,:] = f5[:,llc*1:llc*2]
    arr_tiles[12,:] = f5[:,llc*2:]
    
    if 1 == 0:
        plt.figure()
        plt.imshow(arr_tiles[0], origin='lower')
        plt.figure()
        plt.imshow(arr_tiles[1], origin='lower')
        plt.figure()
        plt.imshow(arr_tiles[2], origin='lower')
    
    return arr_tiles


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""ECCO v4 Python: Dataset Utililites

This module includes utility routines that operate on the Dataset or DataArray Objects 

.. _ecco_v4_py Documentation :
   https://github.com/ECCO-GROUP/ECCOv4-py
"""

from __future__ import division
import numpy as np
import xarray as xr

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


def minimal_metadata(ds):
    """

    This routine removes some of the redundant metadata that is included with the ECCO v4 netcdf tiles from the Dataset object `ds`.  Specifically, metadata with the tags `A` through `Z` (those metadata records) that describe the origin of the ECCO v4 output.    

    Parameters
    ----------
    ds : Dataset
        An `xarray` Dataset object that was created by loading an 
        ECCO v4 tile netcdf file

    
        
    """

    print 'Removing Dataset Attributes A-Z\n'
    # generate a list of upper case letters in teh alphabet
    myDict= map(chr, range(65, 91))

    for key, value in ds.attrs.iteritems():
        if key in myDict: 
            del ds.attrs[key]
         



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""ECCO v4 Python: tile_exchange.py

This module includes utility routines adding halos to individual llc tiles arrays. 

.. _ecco_v4_py Documentation :
   https://github.com/ECCO-GROUP/ECCOv4-py
"""
from __future__ import division
import numpy as np
import xarray as xr
from copy import deepcopy




def append_border_to_tile(ref_arr, tile_index, point_type, llcN, **kwargs):
    """

    This routine takes an array corresponding to an llc tile (the reference array) and appends one of three possible partial "halos":
    1) a new row along the "top" of the array (new 'v' points) 
    2) a new column to the "right" of the array (new 'u' points)
    3) a new single value in a "corner (g) point" of the array (new 'g' point).
    
    After the routine the new array will have dimension depending on where the variable is on the Arakawa-C grid
    1) (nx, ny+1) for appending values from variables on 'v' points 
    2) (nx+1, ny) for appending values from variables on 'u' points  
    3) (nx+1, ny+1) for appending values from variables on 'g' points
    
    Appending new values from adjacent neighbors is only permitted for tiles that connect to the "top", "right", and/or "corner (g) point" of the reference tile based on the default llc tile layout. 

    For example, the tile to the "top" of tile 3 is tile 7 because tile 7 connects to the "top" of tile 3.  Tile 9 is the the "right" of tile 5 because tile 5 connects to the "right" of tile 5. Some tiles have are no adjacent neighbors to the right or top, or in the corner: e.g., tiles 10 and 13 has no tile to the right
    
    One use of this routine is to complete a `grid` tile so that it contains all of the parameters that describe the location and distances associated  with the (ni, nj) tracer grid cells associated with the same tile. These include the (ni+1, nj+1) XG and YG coordinates that define the corners of its tracer grid cells.
    
    In the ECCO v4 netcdf tile files indexing of variables begins with the leftmost and bottommost points.  For variables on:
    'u' points : X[0,0] is at the bottom leftmost 'u' point to the  left of the center of the bottom leftmost tracer cell.
    'v' points : V[0,0] is at the bottom leftmost 'v' point below the center of the bottom leftmost tracer cell.
    'g' points : G[0,0] is at the bottom leftmost 'g' point at the   bottom left corner of the bottom leftmost tracer cell 'c' points : C[0,0] is in the middle of the bottom leftmost tracer cell
    
    Note
    ---- 
        The values in the "halo" are taken from adjacent tiles that are passed in kwargs.  The orientation of the reference tile and the adjacent tiles must be in the original 13-tile llc layout (i.e., this routine must be called prior to any rotation or reorientation).

    Note
    ----  
        In the MITgcm the meaning of 'u' and 'v' is not east-west (zonal) and north-south (meridional) in a geographical sense.  Instead 'u' and 'v' are the direction of flow in the 'x' and 'y' directions for the arbitrary local orientation a tracer grid cell.  For the llc grid, the tracer cells in tiles 1-6 are oriented such that their 'x' and 'y' directions are approximately east-west and north-south, respectively.  However, tiles 8-13 are rotated by 90 degrees relatively to tiles 1-7 (the 'x' direction in tiles 8-13 is the negative 'y' direction of tiles 1-6 and the 'y' directin in tiles 8-13 is the positive 'x' direction in tiles 1-6.  The polar cap tile 7 is a special case.  There is no rotation of tile 7 that will align its 'x' and 'y' grid to be approximately zonal or meridional in a geographical sense.
    
    
    Parameters
    ----------
    ref_arr : ndarray
        A string with the directory of the binary file to open
    tile_index : int
        the index of the reference tile [1-13].
    point_type : string
        The type of partial halo to put on the reference tile, one of 'u','v',or 'g'.  
        'u' add one column to the right of the array
        'v' add one row to the top of the array
        'g' add one column to the right of the array and one row to the top of the array
    llcN : int
        the size of the llc grid.  For ECCO v4, we use the llc90 domain so `llcN` would be 90
    **kwargs
        right : ndarray
            the unrotated tile along the "right" edge. for 'g' and 'u' points
            for tile 2, tile 5
            for tile 9, tile 10
            for tile 7, tile 8
        top : ndarray
            the unrotated tile along the "top" edge. for 'g' and 'v' points
            for tile 2, tile 3
            for tile 9, tile 12
            for tile 7, tile 11 (rotated)
        corner : ndarray
            the unrotated tile in the "top right corner" for 'g' points
            for tile 2, tile 6
            for tile 9, tile 13
            for tile 7, there is none

    Returns
    -------
    ndarray
        a new array that has the halo points around it.

    Examples
    --------
    Add a halo on `ref_array` to the top row and right column at the 'g' points.

    ex 1. This tile has adjacent tiles to its right top and top right corner.

    >>> newarr = append_border_to_tile(ref_arr, tile_index, 
                                      'g', llcN,
                                      right = right_arr, 
                                      top = top_arr, 
                                      corner = corner_arr)

    ex 2. Add a halo on `ref_array` to the right column at the 'u' points.
    
    >>> newarr = append_border_to_tile(ref_arr, tile_index,
                                       'u', llcN, right = right_arr)
    """

    add_right  = False
    add_top    = False
    add_corner = False
    
    right_arr = []
    top_arr = []
    corner_arr = []
    new_arr = []
    
    #%%
    for key in kwargs:
        if key == "right":
            if isinstance(kwargs[key], xr.core.dataarray.DataArray):
                add_right = True
                right_arr = kwargs[key]
        elif key == "top":
            if isinstance(kwargs[key], xr.core.dataarray.DataArray):
                add_top = True
                top_arr = kwargs[key]
        elif key == "corner" :
            if isinstance(kwargs[key], xr.core.dataarray.DataArray):
                add_corner = False
                corner_arr = kwargs[key]

    if add_right == False and add_top == False and add_corner == False:
        raise ValueError('You need to have at least one side to append, \
                         returning an empty array')

    ni = llcN
    nj = llcN
    
    #print ni, nj, llcN
    
    new_arr = np.copy(ref_arr)
    """
    expand the size of the array so that it can hold the new border values
    fill border values with nans.  padding goes like (before, after)
    
    ex 1: two dimensions, pad one column along the end of the j dimension 
             pad_width=((0,0), (0,1))
    ex 2: three dimensions, pad one column along the end of the i dimension 
             pad_width=((0,0), (0,1), (0,0))
    ex 3: three dimensions, pad one column at the start of the i dimension
          and one at the end of the j dimension
             pad_width=((0,0), (1,0), (0,1))
    """
    num_dims = new_arr.ndim

    if point_type == 'g':            
        pad_i = 1 # add one row (y)
        pad_j = 1 # add one column (x)
    elif point_type == 'v':
        pad_i = 1 # add one row (y)
        pad_j = 0 # do not add one column (x)
    elif point_type == 'u':
        pad_i = 0 # do not add one row (x)
        pad_j = 1 # add one column (x)
        
    # print 'num dims ', num_dims


    if num_dims == 2:
        new_arr=np.pad(new_arr, 
            pad_width=((0,pad_i), (0,pad_j)), 
            mode='constant', constant_values = np.nan)

    elif num_dims == 3:
        new_arr=np.pad(new_arr, 
            pad_width=((0,0),(0,pad_i), (0,pad_j)),
            mode='constant', constant_values = np.nan)

    elif num_dims == 4:
        new_arr = np.pad(new_arr, 
            pad_width=((0,0),(0,0),(0,pad_i), (0,pad_j)), 
            mode='constant', constant_values = np.nan)

    else:
        raise ValueError('appending rows or columns in this routine requires \
                         2,3, or 4D arrays')
    
    
    if (tile_index == 1 or tile_index ==  2 or tile_index == 8 or 
        tile_index == 9 or tile_index == 10):
        
        if add_right:
            new_arr[...,0:nj,-1] = right_arr[...,:,0]

        if add_top :
            new_arr[...,-1,0:ni] = top_arr[..., 0,:]

        if add_corner :
            new_arr[...,-1,-1]   = corner_arr[..., 0,0]
     
    elif tile_index == 3:
        if add_right:
            new_arr[...,0:nj,-1] = right_arr[..., :,0]

        if add_top:
            # tricky because tile 7 is rotated relative to 3
            if point_type == 'g':            
                new_arr[...,-1,1:ni+1] = top_arr[..., ::-1,0]                    
            elif point_type == 'v':
                new_arr[...,-1,:] = top_arr[..., :,0]
                
    elif (tile_index == 4 or tile_index == 5):
        if add_right:
            if point_type == 'g':
                new_arr[...,1:nj+1,-1] = right_arr[..., 0,::-1]
                
            elif point_type == 'u':
                new_arr[...,-1] = right_arr[..., 0,::-1]
                
        if add_top:
            new_arr[...,-1,0:ni] = top_arr[...,0,:]
                
    elif tile_index == 6:
        if add_right:
            if point_type == 'g':
                new_arr[...,1:nj+1,-1] = right_arr[...,0,::-1]
                
            elif point_type == 'u':
                new_arr[...,-1] = right_arr[...,0,::-1]
                
        if add_top:
            new_arr[...,-1,0:ni] = top_arr[...,0,:]
                
    elif tile_index == 7:
        if add_right:
            new_arr[...,0:nj,-1] = right_arr[...,:,0]

        if add_top:
            if point_type == 'g':
                # tricky because tile 11 is rotated relative to 7
                new_arr[...,-1,1:ni+1] = top_arr[...,::-1,0]

            elif point_type == 'v':
                new_arr[...,-1,:] = top_arr[...,::-1,0]
    
    elif (tile_index == 11 or tile_index == 12 or 
          tile_index == 13):
        if add_right:
            new_arr[...,0:ni,-1] = right_arr[...,:,0]
            
        if add_top:
            if point_type == 'g':
                new_arr[...,-1,1:nj+1] = top_arr[...,::-1,0]

            elif point_type == 'v':
                new_arr[...,-1,:] = top_arr[...,::-1,0]

        if add_corner :
            """             
            for tiles 12 and 13 the 'corner g point' is
            in the top left point, unlike tiles 1-6 for which the 
            'corner' g point is the top right point.  tile 11 does not
            have a top left point because of the orientation of tile 7.
            """                
            new_arr[...,-1,0] = corner_arr[...,0,0]
        
    #%%
    return new_arr
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





def add_borders_to_DataArray_V_points(da_u, da_v):
    """
    A routine that adds a column to the "top" of the 'v' point 
    DataArray da_v so that every tracer point in the tile 
    will have a 'v' point to the "north" and "south"

    After appending the border the length of da_v in y will
    be +1 (one new row)
    
    This routine is pretty general.  Any tiles can be in the da_u and 
    da_v DataArrays but if the tiles to the "top" of the da_v tiles
    are not available then the new rows will be filled with nans.

    Parameters
    ----------
    da_u : DataArray
        The `DataArray` object that has tiles of a u-point variable
        Tiles of the must be in their original llc layout.

    da_v : DataArray
        The `DataArray` object that has tiles of the v-point variable that 
        corresponds with da_u.   (e.g., VVEL corresponds with UVEL)
        Tiles of the must be in their original llc layout.
       
    Returns
    -------
    da_v_new: DataArray
        a new `DataArray` object that has the appended values of 'v' along
        its top edge.  The lon_u and lat_u coordinates are lost but all
        other coordinates remain.
    """
    
    #%%
    # the j_g dimension will be incremented by one.
    j_g = np.arange(1, len(da_v.j_g)+2)
    
    # the i dimension is unchanged.
    i = da_v['i'].values
    llcN = len(i)
    
    # the k dimension, if it exists, is unchanged.
    if 'k' in da_v.dims:
        nk = len(da_v.k)  
        k = da_v['k'].values
    else:
        nk = 0

    # the time dimension, if it exists, is unchanged
    if 'time' in da_v.dims:
        time = da_v['time'].values
            
    #%%
    #print "\n>>> ADDING BORDERS TO V POINT TILES\n"
    
    # tiles whose tile to the top are rotated 90 degrees counter clockwise
    rot_tiles = {3, 7, 11, 12, 13}

    # the new arrays will be one longer in the j direction, +1 column
    pad_i = 1 # pad by one in y (one new row)
    pad_j = 0 # do not pad in x 

    # set the number of processed tiles counter to zero    
    num_proc_tiles = 0

    # find the number of non-tile dimensions
    if 'tile' in da_v.dims:
        num_dims = da_v.ndim - 1
    else:
        num_dims = da_v.ndim

    # loop through all tiles in da_v
    for tile_index in da_v.tile.values:
        
        # find out which tile is to the top of this da_v tile
        right_tile_index, top_tile_index, corner_tile_index = \
            get_llc_tile_border_mapping(tile_index)

        # if 'tile' exists as a dimension, select and copy this da_v tile
        if 'tile' in da_v.dims:
            ref_arr = deepcopy(da_v.sel(tile=tile_index))
        else:
            # otherwise we have a single da_v tile so make a copy of it
            ref_arr = deepcopy(da_v)

        # the append_border flag will be true if we have a tile to the top.
        append_border = False
        
        #print '\ncurrent da_v tile ', tile_index
        #print 'top tile index ', top_tile_index
        
        # check to see if there even is a tile to the top of da_v tile_index
        if top_tile_index > 0:
            #print 'there is a tile to the top of da_v tile ', tile_index

            # determine whether the tile to the top is rotated relative 
            # to da_v tile_index. if so we'll need da_u!
            if tile_index in rot_tiles:
                #print 'append with da_u tile ', top_tile_index

                if top_tile_index in da_u.tile.values:
                    #print 'we have da_u tile ', top_tile_index                      
                    
                    # see if we have multiple da_u tiles. 
                    if len(da_u.tile) > 1:
                        # pull out the one we need.
                        top_arr = da_u.sel(tile=top_tile_index)
                        append_border = True
                        #print 'appending from da_u tile ', top_tile_index
                    # there is only one da_u tile 
                    elif da_u.tile == top_tile_index:
                        # it is the one we need.
                        top_arr = da_u
                        append_border = True                        
                        #print 'appending from da_u tile ', top_tile_index                        
                    # something may have gone wrong.
                    else:
                        print 'something is wrong with the da_u tile'
                        
                 # if we do not have the da_u tile, then we can't append!
                else:
                   print 'we do not have da_u tile ', top_tile_index

            # the values to append to the top come from another da_v tile
            else:
                #print 'append with da_v tile ', top_tile_index

                # see if we have the required da_v tile
                if top_tile_index in da_v.tile.values:
                    #print 'we have da_v tile ', top_tile_index                      

                    # see if we have multiple da_v tiles
                    if len(da_v.tile) > 1:
                        # pull out the one we need.
                        top_arr = da_v.sel(tile=top_tile_index)
                        append_border = True
                        #print 'appending from da_v tile ', top_tile_index
                    # if we only have one tile then something is wrong because
                    # the tile to the top of this da_v tile cannot be itself
                    #else:
                       # print 'tile to the top cannot be tile_index'
                # we do not have the required da_v tile.
                else:
                    print 'we do not have da_v tile ', top_tile_index

        # there is no tile to the top 
        else:
            print 'there is no tile to the top of da_v tile ', tile_index

        # if we have found a tile to the top we can do the appending
        if append_border:
            new_arr=append_border_to_tile(ref_arr, tile_index,
                                               'v', llcN,
                                               top = top_arr)            

        # if not then we will append an array of nans
        else:
            if num_dims == 2:
                pad = ((0, pad_i), (0, pad_j))
            elif num_dims == 3:
                pad = ((0, 0),     (0, pad_i), (0, pad_j))
            elif num_dims == 4:
                pad = ((0, 0),     (0, 0),     (0, pad_i), (0, pad_j))
            
            new_arr = np.pad(ref_arr, pad_width = pad, mode='constant',
                             constant_values = np.nan)

        # create a new DataArray
        if num_dims == 2:
            new_coords = [('j_g', j_g), ('i', i)]
        elif num_dims == 3 and nk > 0:
            new_coords = [('k', k), ('j_g', j_g), ('i', i)]
        elif num_dims == 3 and nk == 0:
            new_coords = [('time', time),('j_g', j_g), ('i', i)]
        elif num_dims == 4:
            new_coords = [('time', time), ('k', k), ('j_g', j_g), ('i', i)]
            
        tmp_DA = xr.DataArray(new_arr, name = da_v.name, coords=new_coords)

        # give the new DataArray the same attributes as da_v
        tmp_DA.attrs = da_v.attrs

        # give the new DataArray a tile coordinate
        tmp_DA.coords['tile'] = tile_index
    
        # increment the number of processed tiles counter by one
        num_proc_tiles += 1
        
        # set da_v_new equal to tmp_DA if this is the first processed tile
        if num_proc_tiles == 1:
            da_v_new = tmp_DA
        # otherwise, concatentate tmp_DA with da_v_new along the 'tile' dim
        else:
            da_v_new = xr.concat([da_v_new, tmp_DA],'tile')

        # reset tmp_DA
        tmp_DA = []

    # add all time coordinates to_da_v_new from da_v.
    for idx, var in enumerate(da_v.coords):
        if 'tim' in var:
            da_v_new[var] = da_v[var]

    da_v_new.attrs['padded'] = True            
    #%%
    return da_v_new
    #%%


def add_borders_to_DataArray_U_points(da_u, da_v):
    """
    A routine that adds a column to the "right" of the 'u' point 
    DataArray da_u so that every tracer point in the tile 
    will have a 'u' point to the "west" and "east"

    After appending the border the length of da_u in x 
    will be +1 (one new column)
    
    This routine is pretty general.  Any tiles can be in the da_u and 
    da_v DataArrays but if the tiles to the "right" of the da_u tiles
    are not available then the new rows will be filled with nans.
    
    Parameters
    ----------
    da_u : DataArray
        The `DataArray` object that has tiles of a u-point variable
        Tiles of the must be in their original llc layout.

    da_v : DataArray
        The `DataArray` object that has tiles of the v-point variable that 
        corresponds with da_u.   (e.g., VVEL corresponds with UVEL)
        Tiles of the must be in their original llc layout.
       
    Returns
    -------
    da_u_new: DataArray
        a new `DataArray` object that has the appended values of 'u' along
        its right edge.  The lon_u and lat_u coordinates are lost but all
        other coordinates remain.
    """
    
    #%%
    # the i_g dimension will be incremented by one.
    i_g = np.arange(1, len(da_u.i_g)+2)
    
    # the j dimension is unchanged.
    j = da_u['j'].values
    llcN = len(j)
    
    # the k dimension, if it exists, is unchanged.
    if 'k' in da_u.dims:
        nk = len(da_u.k)  
        k = da_u['k'].values
    else:
        nk = 0

    # the time dimension, if it exists, is unchanged
    if 'time' in da_u.dims:
        time = da_u['time'].values
            
    #%%
    #print "\n>>> ADDING BORDERS TO U POINT TILES\n"
    
    # tiles whose tile to the right are rotated 90 degrees counter clockwise
    # to add borders from tiles 4, 5, or 6 we need to use the da_v fields
    rot_tiles = {4, 5, 6}

    # the new arrays will be one longer in the j direction, +1 column
    pad_j = 1 # add one to the second dimension (x)
    pad_i = 0 # we do not pad in first dimension (y)    

    # set the number of processed tiles counter to zero    
    num_proc_tiles = 0

    # find the number of non-tile dimensions
    if 'tile' in da_u.dims:
        num_dims = da_u.ndim - 1
    else:
        num_dims = da_u.ndim
            
    # loop through all tiles in da_u
    for tile_index in da_u.tile.values:
        
        # find out which tile is to the right of this da_u tile
        right_tile_index, top_tile_index, corner_tile_index = \
            get_llc_tile_border_mapping(tile_index)
        
        # if 'tile' exists as a dimension, select and copy the proper da_u tile 
        if 'tile' in da_u.dims:
            ref_arr = deepcopy(da_u.sel(tile=tile_index))
        else:
            # otherwise we have a single da_u tile so make a copy of it
            ref_arr = deepcopy(da_u)

        # the append_border flag will be true if we have a tile to the right.
        append_border = False
        
        #print '\ncurrent da_u tile ', tile_index
        #print 'right tile index ', right_tile_index
        
        # check to see if there even is a tile to the right of da_u tile_index
        # tiles 10 and 13 don't have one!
        if right_tile_index > 0:
            #print 'there is a tile to the right of da_u tile ', tile_index

            # determine whether the tile to the right is rotated relative 
            # to da_u tile_index. if so we'll need da_v!
            if tile_index in rot_tiles:
                #print 'append with da_v tile ', right_tile_index

                if right_tile_index in da_v.tile.values:
                    #print 'we have da_v tile ', right_tile_index                      
                    
                    # see if we have multiple da_v tiles. 
                    if len(da_v.tile) > 1:
                        # pull out the one we need.
                        right_arr = da_v.sel(tile=right_tile_index)
                        append_border = True
                        #print 'appending from da_v tile ', right_tile_index
                    # there is only one da_v tile 
                    elif da_v.tile == right_tile_index:
                        # it is the one we need.
                        right_arr = da_v
                        append_border = True                        
                        #print 'appending from da_v tile ', right_tile_index                        
                    # something may have gone wrong.
                    else:
                        print 'something is wrong with the da_v tile'
                        
                # if we do not have the da_v tile, then we can't append!
                else:
                    print 'we do not have da_v tile ', right_tile_index

            # the values to append to the top come from another da_u tile
            else:
                #print 'append with da_u tile ', right_tile_index

                # see if we have the required da_u tile
                if right_tile_index in da_u.tile.values:
                    #print 'we have da_u tile ', right_tile_index                      

                    # see if we have multiple da_u tiles
                    if len(da_u.tile) > 1:
                        # pull out the one we need.
                        right_arr = da_u.sel(tile=right_tile_index)
                        append_border = True
                        #print 'appending from da_u tile ', right_tile_index
                    # if we only have one tile then something is wrong because
                    # the tile to the right of this da_u tile cannot be itself
                    else:
                        print 'tile to the right cannot be tile_index'
                # we do not have the required da_u tile.
                else:
                    print 'we do not have da_u tile ', right_tile_index

        # there is no tile to the right 
        #else:
        #    print 'there is no tile to the right of da_u tile ', tile_index

        # if we have found a tile to the right we can do the appending
        if append_border:
            new_arr=append_border_to_tile(ref_arr, tile_index,
                                               'u', llcN,
                                               right = right_arr)            

        # if not then we will append an array of nans
        else:
            if num_dims == 2:
                pad = ((0, pad_i), (0, pad_j))
            elif num_dims == 3:
                pad = ((0, 0),     (0, pad_i), (0, pad_j))
            elif num_dims == 4:
                pad = ((0, 0),     (0, 0),     (0, pad_i), (0, pad_j))
            
            new_arr = np.pad(ref_arr, pad_width = pad, mode='constant',
                             constant_values = np.nan)

        # create a new DataArray
        if num_dims == 2:
            new_coords = [('j', j), ('i_g', i_g)]
        elif num_dims == 3 and nk > 0:
            new_coords = [('k', k), ('j', j), ('i_g', i_g)]
        elif num_dims == 3 and nk == 0:
            new_coords = [('time', time),('j', j), ('i_g', i_g)]
        elif num_dims == 4:
            new_coords = [('time', time), ('k', k), ('j', j), ('i_g',i_g)]
            
        tmp_DA = xr.DataArray(new_arr, name = da_u.name, coords=new_coords)

        # give the new DataArray the same attributes as da_u
        tmp_DA.attrs = da_u.attrs

        # give the new DataArray a tile coordinate
        tmp_DA.coords['tile'] = tile_index
    
        # increment the number of processed tiles counter by one
        num_proc_tiles += 1
        
        # set da_u_new equal to tmp_DA if this is the first processed tile
        if num_proc_tiles == 1:
            da_u_new = tmp_DA
        # otherwise, concatentate tmp_DA with da_u_new along the 'tile' dim
        else:
            da_u_new = xr.concat([da_u_new, tmp_DA],'tile')

        # reset tmp_DA
        tmp_DA = []

    # add all time coordinates to_da_u_new from da_u.
    for idx, var in enumerate(da_u.coords):
        if 'tim' in var:
            da_u_new[var] = da_u[var]

    da_u_new.attrs['padded'] = True            
    #%%       
    return da_u_new
    #%%
    
    
    
def add_borders_to_DataArray_G_points(da_g):
    """
    A routine that adds a row and column to the "top" and "right"
    of the 'g' point DataArray da_g so that every tracer point in the tile 
    will have a 'g' point on all of its corners

    After appending the border the length of da_g will be +1 in both 
    i_g and j_g
    
    
    Parameters
    ----------
    da_ g: DataArray
        The `DataArray` object that has tiles of a g-point variable
        Tiles of the must be in their original llc layout.

       
    Returns
    -------
    da_g_new: DataArray
        a new `DataArray` object that has the appended values of 'g' along
        its top and right edges. 
    """
    
    #%%
    # the i_g and j_g dimensions will be incremented by one.
    llcN = len(da_g.i_g)
    j_g = np.arange(1, len(da_g.j_g)+2)
    i_g = np.arange(1, len(da_g.i_g)+2)


    # the k dimension, if it exists, is unchanged.
    if 'k' in da_g.dims:
        nk = len(da_g.k)  
        k = da_g['k'].values
    else:
        nk = 0

    # the time dimension, if it exists, is unchanged
    if 'time' in da_g.dims:
        time = da_g['time'].values
            
    #%%
    #print "\n>>> ADDING BORDERS TO G POINT TILES\n"
    
    # the new arrays will be one longer in each dimensin
    pad_i = 1 # pad by one in y (one new row)
    pad_j = 1 # do not pad in x 

    # set the number of processed tiles counter to zero    
    num_proc_tiles = 0

    # find the number of non-tile dimensions
    if 'tile' in da_g.dims:
        num_dims = da_g.ndim - 1
    else:
        num_dims = da_g.ndim

    if len(da_g.tile.values) == 1:
        raise ValueError('You can not append to da_g because you only \
                          passed a single tile!')

    # loop through all tiles in da_g
    for tile_index in da_g.tile.values:
        
        # find out which tiles are adjacent 
        right_tile_index, top_tile_index, corner_tile_index = \
            get_llc_tile_border_mapping(tile_index)
        
        ref_arr = deepcopy(da_g.sel(tile=tile_index))

        # the append_border flag will be true if we have a tile to the top.
        append_border_top    = False
        append_border_right  = False 
        append_border_corner = False         
        
        #print '\ncurrent da_g tile ', tile_index
        #print 'top tile index ', top_tile_index
        #print 'right tile index ', right_tile_index
        #print 'corner tile index ', corner_tile_index
        
        top_arr = []
        right_arr = []
        corner_arr = []
 

        if top_tile_index in da_g.tile.values:
            #print 'we have da_g tile to the top ', top_tile_index                      
            top_arr = da_g.sel(tile=top_tile_index)
            append_border_top = True

        if right_tile_index in da_g.tile.values:
            #print 'we have da_g tile to the right ', right_tile_index                      
            right_arr = da_g.sel(tile=right_tile_index)
            append_border_right = True
            
        if corner_tile_index in da_g.tile.values:
            #print 'we have da_g tile in the NE corner ', corner_tile_index                      
            corner_arr = da_g.sel(tile=corner_tile_index)
            append_border_corner = True

        if append_border_top or append_border_right or append_border_corner:
            new_arr=append_border_to_tile(ref_arr, tile_index,
                                          'g', llcN,
                                          top = top_arr,
                                          right = right_arr,
                                          corner = corner_arr)

        # if not then we will append an array of nans
        else:
            if num_dims == 2:
                pad = ((0, pad_i), (0, pad_j))
            elif num_dims == 3:
                pad = ((0, 0),     (0, pad_i), (0, pad_j))
            elif num_dims == 4:
                pad = ((0, 0),     (0, 0),     (0, pad_i), (0, pad_j))
            
            new_arr = np.pad(ref_arr, pad_width = pad, mode='constant',
                             constant_values = np.nan)

        # create a new DataArray
        if num_dims == 2:
            new_coords = [('j_g', j_g), ('i_g', i_g)]
        elif num_dims == 3 and nk > 0:
            new_coords = [('k', k), ('j_g', j_g), ('i_g', i_g)]
        elif num_dims == 3 and nk == 0:
            new_coords = [('time', time),('j_g', j_g), ('i_g', i_g)]
        elif num_dims == 4:
            new_coords = [('time', time), ('k', k), ('j_g', j_g), ('i_g', i_g)]
            
        tmp_DA = xr.DataArray(new_arr, name = da_g.name, coords=new_coords)

        # give the new DataArray the same attributes as da_g
        tmp_DA.attrs = da_g.attrs

        # give the new DataArray a tile coordinate
        tmp_DA.coords['tile'] = tile_index
    
        # increment the number of processed tiles counter by one
        num_proc_tiles += 1
        
        # set da_g_new equal to tmp_DA if this is the first processed tile
        if num_proc_tiles == 1:
            da_g_new = tmp_DA
        # otherwise, concatentate tmp_DA with da_g_new along the 'tile' dim
        else:
            da_g_new = xr.concat([da_g_new, tmp_DA],'tile')

        # reset tmp_DA
        tmp_DA = []

    # add all time coordinates to_da_g_new from da_g.
    for idx, var in enumerate(da_g.coords):
        if 'tim' in var:
            da_g_new[var] = da_g[var]

    da_g_new.attrs['padded'] = True            
            
    #%%            
    return da_g_new
    #%%
    

def get_llc_tile_border_mapping(tile_index):
    """
    Return the tile numbers for the tiles adjacent to the tile at 
    `tile_index` along the 'right','top' and 'corner' in the original llc tile layout (if it exists)
    
    Note
    ----
        In some cases there is no tile in the adjacent location.  For example, there is no tile to the "right" of tile 13 in the original llc tile layout.

    
    Parameters
    ----------
    tile_index : int
        the index of the reference tile [1-13].
    

    Returns
    -------
    right_tile_index, top_tile_index, corner_tile_index : ints
        the indices of the tiles to the right, top, and top right corner of `tile_index`.  If there is no tile in the location, -1 is returned.
        

    """
    if tile_index == 1:
        right_tile_index = 4
        top_tile_index = 2
        corner_tile_index= 5
    elif tile_index == 2:
        right_tile_index = 5
        top_tile_index = 3
        corner_tile_index = 6
    elif tile_index == 3:
        right_tile_index = 6
        top_tile_index = 7
        corner_tile_index = -1 # does not exist 
    elif tile_index == 4:
        right_tile_index = 10
        top_tile_index = 5
        corner_tile_index = -1 # does not exist 
    elif tile_index == 5:
        right_tile_index = 9
        top_tile_index = 6  
        corner_tile_index = -1 # does not exist 
    elif tile_index == 6:
        right_tile_index = 8
        top_tile_index = 7
        corner_tile_index = -1 # does not exist 
    elif tile_index == 7:
        right_tile_index = 8
        top_tile_index = 11
        corner_tile_index = -1 # does not exist 
    elif tile_index == 8:
        right_tile_index = 9
        top_tile_index = 11
        corner_tile_index = 12
    elif tile_index == 9:
        right_tile_index = 10
        top_tile_index = 12
        corner_tile_index = 13
    elif tile_index == 10:
        right_tile_index = -1 # does not exist 
        top_tile_index = 13
        corner_tile_index = -1 # does not exist 
    elif tile_index == 11:
        right_tile_index = 12
        top_tile_index = 3
        corner_tile_index = -1 # does not exist 
    elif tile_index == 12:
        right_tile_index = 13
        top_tile_index = 2
        corner_tile_index = 3 # yes, corner is "top left"
    elif tile_index == 13:
        right_tile_index = -1 # does not exist 
        top_tile_index = 1
        corner_tile_index = 2 # yes, corner is "top left"
    
    return right_tile_index, top_tile_index, corner_tile_index   
 
            
def add_borders_to_GRID_tiles(gds):
    """
    A routine that appends the partial halo for all grid tile variables.  
    
    Parameters
    ----------
    gds : Dataset
        the `Dataset` object that has one or more grid ECCO v4 tile.
        The tiles of the grid Dataset must be in their original layout.
    

    Returns
    -------
    gds_pad : Dataset
        a new `Dataset` object for the ECCO v4 grid that has been padded
        on the top and right sides for 'v' and 'u' point variables, 
        respectively and on the top, top right corner, and right sides for
        'g' point variables.  'c' point and vertical grid parameters are
        untouched.
        
        gds_pad also has the new attribute, padded=True
    """            
    #%%
    print "\n>>> ADDING BORDERS TO GRID TILES\n"

    # C points
    ## no borders added to C points.  We're adding borders AROUND c points

    print " --- 'g' points"
    # G points
    XG_b  = add_borders_to_DataArray_G_points(gds.XG).to_dataset();

    YG_b  = add_borders_to_DataArray_G_points(gds.YG).to_dataset();    

    RAZ_b = add_borders_to_DataArray_G_points(gds.RAZ).to_dataset();        

    print " --- 'u' points"
    # U points
    DXC_b    = add_borders_to_DataArray_U_points(gds.DXC, 
                                                 gds.DYC).to_dataset()

    DYG_b    = add_borders_to_DataArray_U_points(gds.DYG, 
                                                 gds.DXG).to_dataset()

    hFacW_b  = add_borders_to_DataArray_U_points(gds.hFacW, 
                                                 gds.hFacS).to_dataset()

    land_u_b = add_borders_to_DataArray_U_points(gds.land_u, 
                                                 gds.land_v).to_dataset()   
  
    print " --- 'v' points"
    # V points
    DYC_b    = add_borders_to_DataArray_V_points(gds.DXC, 
                                                 gds.DYC).to_dataset()

    DXG_b    = add_borders_to_DataArray_V_points(gds.DYG, 
                                                 gds.DXG).to_dataset()

    hFacS_b  = add_borders_to_DataArray_V_points(gds.hFacW, 
                                                 gds.hFacS).to_dataset()

    land_v_b = add_borders_to_DataArray_V_points(gds.land_u, 
                                                 gds.land_v).to_dataset()   
    # now merge 
    
    # pull off the vertical parameters
    gds_tmp_Z = gds[['RC','RF','DRC','DRF']]
    
    # drop all of the parameters that we just added borders to and the vertical
    # grid parameters
    gds_tmp = gds.drop(['DYC','DXG', 'hFacS',
                        'DXC', 'DYG','hFacW','XG','YG',
                        'RAZ','RC','RF','DRC','DRF'])
    
    # merge the old c-point grid parameters and the new padded 'g','u', and 'v'
    # point parameters, and the vertical grid parameters
    gds_pad = xr.merge([gds_tmp, XG_b, YG_b, RAZ_b, DXC_b, DYG_b, hFacW_b, 
                        land_u_b, DYC_b, DXG_b, hFacS_b, land_v_b, gds_tmp_Z])
    
    # give the new grid Dataset the same parameters
    gds_pad.attrs = gds.attrs
    gds_pad.attrs['padded'] = True
    
    #%%
    return gds_pad
    #%%


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon Jul  3 16:11:15 2017

@author: ifenty
"""
from __future__ import division
import numpy as np
import matplotlib.pylab as plt
import xarray as xr
import time
from copy import deepcopy
import glob
from numpy import *
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


def load_all_tiles_from_netcdf(data_dir, var, var_type, **kwargs):

    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    # Reads and concatenates all of the 13 netcdf ecco tiles as an xarray 
    # DataSet object.  A new dimension is added, 'i0', corresponding to 
    # tile number (1, 2, ... 13)
    # * can handle 2D output: lat, lon [e.g., a time-mean surface field]    
    # * can handle 3D output: lat, lon, time [e.g., SSH]
    # * can handle 4D output: lat, lon, depth, time [e.g., THETA]    
    #
    # tiles are kept in their original llc orientation (i.e., tiles 8-13 are
    # rotated by 90 relative to tiles 1-6).
    #
    # data_dir : the directory with the netcdf files
    #    
    # var      : name of the variable in the netcdf tile file.  For example, 
    #            var is 'GRID' for the grid files [GRID.0001.nc, GRID.0002.nc, ...
    #            GRID.0013.nc] and var is 'THETA' for theta files [TEHTA.0001.nc
    #            THETA.0002.nc, ... THETA.00013.nc]
    #
    # var_type : variables can be of one of 5 types.
    #            'c' for variables situated at the cell center [e.g., THETA, SALT, SSH]
    #            'g' for variables situated at the cell corners 
    #            'u' for variables situated at the cell u-velocity points [e.g., UVEL]
    #            'v' for variables situated at the cell v-velocity points [e.g., VVEL]
    #            'grid' - a special case for the netcdf GRID files that contain
    #                     llc grid parameters variables.  GRID files contain
    #                     variables situated on a mix of 'c','g','u', and 'v' 
    #                     points.  
    #
    # var_type also determines how the coordinates of the variable are renamed.
    # The default variable dimension names in the llc tile files are not very
    # descriptive (e.g., 'i1','i2','i3').  
    # The coordinates corresponding with horizontal dimensions are renamed 
    # depending on var_type.  To wit,
    #  'i'   and 'j'   for 'c' point variables 
    #  'i_g' and 'j_g' for 'g' point  variables
    #  'i_g' and 'j'   for 'u' point variables
    #  'i'   and 'j_g' for 'v' point variables
    #
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    # By default print messages to the screen when loading tile files
    less_output = False

    for key in kwargs:
        if key == "less_output":
            less_output = kwargs[key]
    #%%
 
    if less_output == False:
        print "\n>>> LOADING TILES FROM NETCDF\n"
    start = time.time()
    
    for tile_index in range(1, 14):  
        ds_tile = load_tile_from_netcdf(data_dir, var, var_type,
                                                tile_index, **kwargs)

        if tile_index == 1:
            ds= {str(tile_index):ds_tile}
        else:
            ds[str(tile_index)]=ds_tile

        ds_tile = []

    
    end = time.time()
    if less_output == False:
        print 'total file load time ', end-start, 's'
        print 'concatenated all tiles.  this can take a few minutes....'

    start = time.time()    
    ds = xr.concat([ds['1'], ds['2'], ds['3'],
                    ds['4'], ds['5'], ds['6'],
                    ds['7'], ds['8'], ds['9'],
                    ds['10'],ds['11'], ds['12'],
                    ds['13']], 'tile')
    end = time.time()

    if less_output == False:
        print 'finished concatenating.  time =', end-start, 's'
        
    #%%
    if var == 'GRID':
        # since we have concatenated 14 tiles along the 'tile' dimension
        # the vertical grid parameters now have a tile dimension.  This
        # is unncessary since vertical grid parameters are the same across
        # all tiles.  The code below drops the 'tile' dimension from the
        # vertical coordinates.
        
        # save the metadata on ds, we'll use it later.
        orig_attrs = ds.attrs

        # Go through each of these 1D coordinates and drop the 'tile'
        # dimension        
        #RB = ds.RB[0,:].drop('tile')
        RC = ds.RC[0,:].drop('tile')
        RF = ds.RF[0,:].drop('tile')
        DRC = ds.DRC[0,:].drop('tile')
        DRF = ds.DRF[0,:].drop('tile')
        
        # drop these coordinates from ds
        #ds = ds.drop(['RB', 'RC','RF','DRC','DRF']); 
        ds = ds.drop(['RC','RF','DRC','DRF']); 
        
        # add the coordinates back in now that we have dropped the 'tile'
        # dimension
        #ds = xr.merge([ds, RB, RC, RF, DRC, DRF])
        ds = xr.merge([ds, RC, RF, DRC, DRF])
        
        # reset the metadata on ds.
        ds.attrs = orig_attrs

    print "Finished loading all 13 tiles of " + var        

    return ds
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    
        
def load_tile_from_netcdf(data_dir, var, var_type, tile_index, **kwargs):

    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    # Reads one of the 13 netcdf ecco tiles as an xarray DataSet object
    # Tiles 1-6, and 8-13 are rotated so that they align in lat/lon 
    # Tile 7 (Arctic Cap)  is rotated to line up with Tile 11 (N. America)
    # * can handle 2D output: lat, lon, time [e.g., SSH]
    # * can handle 3D output: lat, lon, depth, time [e.g., THETA]
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    # by default, we will drop the tile's record of land mask, grid cell
    # area, and lat and lon coordinates.  this information is redundant 
    # because the ECCO grid has all of this information.  keeping it around
    # just makes for very messy Datasets
    keep_landmask_and_area = False
    
    #%%
    less_output = False
    k_subset = []
    for key in kwargs:
        if key == "keep_landmask_and_area":
            keep_landmask_and_area = kwargs[key]
        if key == "k_subset":
            k_subset = kwargs[key]
        if key == "less_output":
            less_output = kwargs[key]
        else:
            print "load_tile_from_netcdf unrecognized argument ", key
    #%%
    # construct the netcdf file name based on the variable name and the
    # the tile index
    fname = (data_dir + var + '.' + str(tile_index).zfill(4) + '.nc')

    if less_output == False:
        print 'loading ' + fname

    # load the netcdf file using xarray.  
    # xarray automatically converts the netcdf file into a Dataset object
    # using the netcdf metadata

    # check to see if file exists.    
    file = glob.glob(fname)
    if len(file) == 0:
        raise IOError(fname + ' not found ')

    ds = xr.open_dataset(fname)

    # finally, give this Dataset the 'tile' coordinate with current tile_index
    ds.coords['tile'] = tile_index
    # xarray doesn't do a perfect job of interpreting the information in the
    # netcdf files.  To improve unsability we need to make some changes to the
    # new Dataset object.
    #
    # first, change the type of all coordinate index variables from float64 to 
    # int64.  Coordinate index variables are discrete [1, 2, ... n] 
    # so integers are more logical than floats.
    # -- loop through all dimensions and change the type to int
    #for key, value in ds.dims.iteritems():
    #    ds[key] = ds[key].astype(np.int)

    # second, check to see if we are loading a GRID tile files.  GRID tiles
    # need special treatment because they are a mix of horizontal and vertical
    # parameters on a mix of Arakawa C-grid points ('c','g','u', and 'v')

    if var != 'GRID':
        # we do not have a GRID tile file, we have either a 4D file like THETA
        # which has four dimensions: time, depth, y, and x, or a 3D file like
        # or a 3D file like SSH which has three dimensions: time, y, and x
        num_dims = len(ds.dims)

        # the model time step 'timstep' is loaded as a variable instead of as
        # a coordinate label.  Here, I make 'timstep' a coordinate label.
        # -- step 1, add a corordinate to the Dataset with the name 'timestep'
        #            with the value of the original 'timstep' variable
        ds.coords['timestep'] = ('time', ds['timstep'].values)
        # -- step 2, drop 'timstep' as a variable from the Dataset
        ds = ds.drop('timstep')
        
        ds = ds.expand_dims('tile')

        # give the coordinates more meaningful names depending on where on the
        # Arakawa C-grid the variable is situated.
        if num_dims == 4 :
            if len(k_subset) > 0 :
                ds = ds.sel(i2=k_subset)

            if var_type == 'c':
                ds=ds.rename({'i1':'time','i2':'k', 'i3':'j','i4':'i'})                    
                ds = ds.transpose('time','tile','k','j','i')
                
            elif var_type == 'g':
                ds=ds.rename({'i1':'time','i2':'k', 'i3':'j_g','i4':'i_g'})                                    
                ds = ds.transpose('time','tile','k','j_g','i_g')
            elif var_type == 'u':
                ds=ds.rename({'i1':'time','i2':'k', 'i3':'j','i4':'i_g'})                                    
                ds = ds.transpose('time','tile','k','j','i_g')
                
            elif var_type == 'v':
                ds=ds.rename({'i1':'time','i2':'k', 'i3':'j_g','i4':'i'})                                    
                ds = ds.transpose('time','tile','k','j_g','i')
                
            else:
                print 'to give sensible coordinate names \'var_type\' must be one of: c, g, u or v'

        # give the coordinates more meaningful names depending on where on the
        # Arakawa C-grid the variable is situated.
        elif num_dims == 3 :
            if var_type == 'c':
                ds = ds.rename({'i1':'time','i2':'j','i3':'i'})  
                ds = ds.transpose('time','tile','j','i')

            elif var_type == 'g':
                ds=ds.rename({'i1':'time','i2':'j_g','i3':'i_g'})                                    
                ds = ds.transpose('time','tile','j_g','i_g')
                
            elif var_type == 'u':
                ds=ds.rename({'i1':'time','i2':'j','i3':'i_g'})                                    
                ds = ds.transpose('time','tile','j','i_g')
                
            elif var_type == 'v':
                ds=ds.rename({'i1':'time','i2':'j_g','i3':'i'})     
                ds = ds.transpose('time','tile','j_g','i')
                
            else:
                print 'to give sensible coordinate names \'var_type\' must be one of: c, g, u or v\n'
            
        # ECCO v4 Model output on the tile files include four auxillary fields,
        # corresponding with the land mask, area (m^2), longitude, and latitude
        # at the location of the variable.  The fields correspond to the same
        # location on the Arakawa C-grid as the variable.  For example, 'lon'
        # corresponds to the longitude where variable is situated on the
        # Arakawa C-grid.  To make the location of these auxillary fields
        # unambiguous, a suffix is added based on where on the Arakawa C-grid
        # it is locations.
            
        # By default, we do not keep the redundant land mask and cell area
        # information.
        if keep_landmask_and_area:
            print 'Keeping land mask and area!\n'
            if 'land' in ds.variables:
                new_land_name = 'land_' + var_type
                ds = ds.rename({'land':new_land_name})
            if 'area' in ds.variables:
                new_area_name = 'area_' + var_type
                ds = ds.rename({'area':new_area_name})

        # drop these two variables 
        ds=ds.drop(['land', 'area'])
        
        if 'thic' in ds.variables:
            ds = ds.drop(['thic'])
            
        if 'lon' in ds.variables:
            new_lon_name = 'lon_' + var_type
            ds = ds.rename({'lon':new_lon_name})
        if 'lat' in ds.variables:
            new_lon_name = 'lat_' + var_type
            ds = ds.rename({'lat':new_lon_name})
                    

    else:
        # we do have a GRID tile file which requires special treatment.
        
        # First, save the attributes (metadata) of the Dataset.  
        # We'll need it later.
        orig_attrs = ds.attrs

        # find the number of vertical levels (ECCO v4 has 50 levels)
        nk = len(ds.RF)
        
       
        
        # Now give the Dataset helpful coordinate dimension
        # names instead of using 'i1','i2','i3'.
        
        # Start with coordinates for grid parameters at the grid cell "top":
        # (1) the depth of the cell top (RF) and (2) cell center distance (DRC)
        # -- step 1, create a new Dataset with just these two parameters
        GRID_ZT = xr.merge([ds.RF, ds.DRC])
        # -- step 2, rename the vertical coordinate to from 'i1' to 'k_g'
        GRID_ZT = GRID_ZT.rename({'i1':'k_g'})

        # Now coordinates for vertical grid parameters at the cell center 
        # -- step 1, create a new temporary Dataset         
        GRID_ZC = xr.merge([ds.RC, ds.DRF])
        # -- step 2, give proper coordinate names
        GRID_ZC = GRID_ZC.rename({'i1':'k'})

        # Now change the coordinates for grid parameters that are 
        # situated on the 'c', 'g', 'u' and 'v points
        # -- step 1, create a new temporary Dataset with params on 'G' points
        GRID_G = ds[['XG', 'YG','RAZ']]
        # -- step 2, give proper coordinate names
        GRID_G= GRID_G.rename({'i2':'j_g', 'i3':'i_g'})

        # -- step 3, create a new temporary Dataset with params on 'U' points
        GRID_U = ds[['DXC','DYG','hFacW']]
        # -- step 4, give proper coordinate names
        GRID_U= GRID_U.rename({'i1':'k','i2':'j', 'i3':'i_g'})

        
        # -- step 5, create a new temporary Dataset with params on 'V' points
        GRID_V = ds[['DYC','DXG','hFacS']]
        # -- step 6, give proper coordinate names
        GRID_V= GRID_V.rename({'i1':'k','i2':'j_g', 'i3':'i'})
        
        # -- step 7, create a new temporary Dataset with params on 'C' points
        GRID_C = ds[['XC','YC','RAC','Depth','AngleCS','AngleSN','hFacC']]
        # -- step 8, give proper coordinate names
        GRID_C = GRID_C.rename({'i1':'k','i2':'j', 'i3':'i'})        

        
        # Now create a new Dataset for the vertical grid parameter
        # RB: the depth (in meters) of the bottom of the grid cell 
        # -- step 1, create an empty array of length equal to the number of
        #            vertical levels
        #tmp = np.zeros(nk)
        # -- step 2, copy over all the depth coordinates of the grid cell top
        #            starting from k=2 (the "top" of the second grid cell
        #            is the "bottom " of the first grid cell)
        #tmp[0:-1] = GRID_ZT.RF[1:]
        # -- step 3, calculate and add the depth of the deepest grid cell
        #            the depth of the deepest grid cell is the depth of the 
        #            second from the deepest grid cell plus the height of the
        #            the last grid cell
        #tmp[-1] = tmp[-2]-ds.DRF[-1]
        # -- step 4, create a new Dataset with this new array and give it a 
        #            vertical coordinate index 'k_l' with values [1., 2... nk].
        #GRID_ZB = xr.Dataset({'RB': (['k_l'], tmp)},
        #                     coords={'k_l':np.arange(1,nk+1)})
        # -- step 5, give the variable the same attributes as RF (depth in m)
        #GRID_ZB.RB.attrs['long_name'] = 'depth of the deepest grid cell'
        #GRID_ZB.RB.attrs['units'] = 'm'

        # next we'll make land/wet masks.  these will prove very useful
        # later.  
        land_v = deepcopy(GRID_V.hFacS)
        land_v.name = 'land_v'
        land_v[:] = np.ceil(land_v.values)
        land_v.attrs['long_name'] = 'land mask on v points'
        
        land_u = deepcopy(GRID_U.hFacW)
        land_u.name = 'land_u'
        land_u[:] = np.ceil(land_u.values)
        land_u.attrs['long_name'] = 'land mask on u points'
        
        land_c = deepcopy(GRID_C.hFacC)
        land_c.name = 'land_c'
        land_c[:] = np.ceil(land_c.values)
        land_c.attrs['long_name'] = 'land mask on c points'

        # now merge our new and improved Datasets together
        ds = xr.merge([GRID_C, land_c, GRID_G, GRID_U, land_u, GRID_V, land_v,
                       GRID_ZT, GRID_ZC]) #, GRID_ZB])

        # give the new merged grid dataset the original attributes
        # (metadata description)
        ds.attrs =  orig_attrs

        # finally, give more useful descriptions of some vertical grid parameters
        # and give the coordinates more meaningful names
        ds.RC.attrs['long_name'] = 'depth of grid cell center'        
        ds.RF.attrs['long_name'] = 'depth of grid cell top'
        # ds.RB.attrs['long_name'] = 'depth of grid cell bottom'                    
        ds.DRF.attrs['long_name'] = 'vertical distance between the grid cell top and bottom'
        ds.DRC.attrs['long_name'] = 'vertical distance between grid cell centers, starting from the ocean surface'
        
      
        
    # loop through each Dataset variable (which, for some reason also includes 
    # coordinates) and give each rotatable variable (dim >= 2) an attribute
    # 'rotated_to_latlon=False' to indicate that the tile has not yet
    # been rotated
    for key, value in ds.variables.iteritems():
        if key not in ds.coords and len(ds[key].shape) >= 2:
            ds.variables[key].attrs['rotated_to_latlon'] = False


    
    #%%
    return ds
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon Jul  3 16:11:15 2017

@author: ifenty
"""
from __future__ import division
import numpy as np
import matplotlib.pylab as plt
import xarray as xr

# import all function from the 'mpl_toolkits.basemap' module available with the 
# prefix 'Basemap'
from mpl_toolkits.basemap import Basemap

# Performs cartographic transformations and geodetic computations.

# The Proj class can convert from geographic (longitude,latitude) to native 
# map projection (x,y) coordinates and vice versa, or from one map projection 
# coordinate system directly to another.
# https://pypi.python.org/pypi/pyproj?
#
import pyresample as pr

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


def plot_tile(tile, **kwargs):

    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    # shows a single llc tile.  
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

    # by default use jet colormap
    user_cmap = 'jet'
    
    # by default do not show a colorbar 
    show_colorbar = False

    # by default the colorbar has no label
    show_cbar_label = False

    # by default take the min and max of the values
    cmin = np.nanmin(tile)
    cmax = np.nanmax(tile)
    
    #%%
    for key in kwargs:
        if key == "cbar":
            show_colorbar = kwargs[key]
        elif key == "user_cmap":
            user_cmap = kwargs[key]
        elif key == "cbar_label":
            cbar_label = kwargs[key]
            show_cbar_label = True
        elif key == "cmin":
            cmin = kwargs[key]
        elif key == "cmax":
            cmax =  kwargs[key]
        else:
            print "unrecognized argument ", key 
    #%%
       
    plt.imshow(tile, vmin=cmin, vmax=cmax, cmap=user_cmap, 
               origin='lower')
    
    plt.xlabel('+x -->')
    plt.ylabel('+y -->')
    
    # show the colorbar
    if show_colorbar:
        cbar = plt.colorbar()
        if show_cbar_label:
            cbar.set_label(cbar_label)

    plt.show()
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    
    
    
def plot_tiles(tiles,  **kwargs):

    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    # this routine plots the 13 llc faces in either the original 'llc' layout
    # or the quasi lat-lon  layout
    # cmin and cmax are the color minimum and maximum
    # max.  'tiles' is a DataArray of a single 2D variable
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

    # by default use jet colormap
    user_cmap = 'jet'
    
    # by default do not show a colorbar 
    show_colorbar = False

    # by default the colorbar has no label
    show_cbar_label = False

    # by default the layout is the original llc 
    layout = 'llc'
    
    # by default take the min and max of the values
    cmin = np.nanmin(tiles)
    cmax = np.nanmax(tiles)
    
    tile_labels = True

    fsize = 9  #figure size in inches (h and w)
    
    rotate_to_latlon = False
    
    for key in kwargs:
        if key == "cbar":
            show_colorbar = kwargs[key]
        elif key == "user_cmap":
            user_cmap = kwargs[key]
        elif key == "cbar_label":
            cbar_label = kwargs[key]
            show_cbar_label = True
        elif key == "layout":
            layout = kwargs[key]
        elif key == "cmin":
            cmin = kwargs[key]
        elif key == "cmax":
            cmax =  kwargs[key]
        elif key == "tile_labels":
            tile_labels = kwargs[key]
        elif key == "fsize":
            fsize = kwargs[key]
        elif key == "rotate_to_latlon":
            rotate_to_latlon = kwargs[key]
        else:
            print "unrecognized argument ", key 
               

    if layout == 'latlon':

        if tile_labels:            
            f, axarr = plt.subplots(4, 4, figsize=(fsize,fsize))
        else:
            fac = 1.1194
            f, axarr = plt.subplots(4, 4, figsize=(fsize*fac,fsize),
                                    gridspec_kw = {'wspace':0, 'hspace':0})

        # plotting of the tiles happens in a 4x4 grid
        # which tile to plot for any one of the 16 spots is indicated with a list
        # a value of negative one means do not plot anything in that spot.
        # the top row will have the Arctic tile.  Where we put the Arctic tile
        # depends on which other tile it is aligned with.  By default, it is
        # aligned with tile 6, which is the second column.  
        tile_order_top_row = [-1, 7, -1, -1]
        
        if type(tiles) == np.ndarray:
            print 'sent a 13 tile array'
        else:
            # we were sent a Dataset or DataArray   
            # if we have defined the attribute 'Arctic_Align' then perhaps
            # the Arctic cap is algined with another tile and therefore it's location
            # in the figure may be different changed.
            if 'Arctic_Align' in tiles.attrs:
                aca = tiles.attrs['Arctic_Align']
                #print 'Arctic Cap Alignment match with tile: ', aca
                
                if  aca == 3: # plot in 1st position, column 1
                    tile_order_top_row = [7, -1, -1, -1]
                elif aca == 6:# plot in 2nd position, column 2
                    tile_order_top_row = [-1, 7, -1, -1]
                elif aca == 8:# plot in 3rd position, column 3
                    tile_order_top_row = [-1, -1, 7, -1]
                elif aca == 11:# plot in 4th position, column 4
                    tile_order_top_row = [-1, -1, -1, 7]
                else:
                    print 'Arctic Cap Alignment is not one of 3, 6, 8, 11.'
                        
        # the order of the rest of the tile is fixed.  four columns each with 
        # three rows.
        tile_order_bottom_rows =[3, 6, 8, 11,
                          2, 5, 9, 12, \
                          1, 4, 10, 13]
        
        # these are lists so to combine tile_orde_first and tile_order_rest 
        # you just add them in python (wierd).  If these were numpy arrays 
        # one would use np.concatenate()
        tile_order = tile_order_top_row + tile_order_bottom_rows
    
    elif layout == 'llc':
        if tile_labels:            
            f, axarr = plt.subplots(5,5, figsize=(fsize,fsize))
        else:
            fac = 1.1194
            f, axarr = plt.subplots(5, 5, figsize=(fsize*fac,fsize),
                                    gridspec_kw = {'wspace':0, 'hspace':0})
            
    
        # plotting of the tiles happens in a 5x5 grid
        # which tile to plot for any one of the 25 spots is indicated with a list
        # a value of negative one means do not plot anything in that spot.
        tile_order = np.array([-1, -1, 11, 12, 13, \
                      -1, 7, 8, 9, 10, \
                      3, 6, -1, -1, -1, \
                      2, 5, -1, -1, -1, \
                      1, 4, -1, -1, -1])
        


    # loop through the axes array and plot tiles where tile_order != -1
    for i, ax in enumerate(axarr.ravel()):
        ax.axis('off')

        cur_tile_num = tile_order[i]
        
        #print i, cur_tile_num
        if cur_tile_num > 0:
            if type(tiles) == np.ndarray:
                cur_tile = tiles[cur_tile_num -1]
            else:
                cur_tile = tiles.sel(tile=cur_tile_num)
            
            if (layout == 'latlon' and rotate_to_latlon and cur_tile_num >7):
                cur_tile = np.copy(np.rot90(cur_tile))
            
            im=ax.imshow(cur_tile, vmin=cmin, vmax=cmax, cmap=user_cmap, 
                         origin='lower')

            ax.set_aspect('equal')
            ax.axis('on')
            if tile_labels:
                ax.set_title('Tile ' + str(cur_tile_num))
                
            ax.get_xaxis().set_visible(False)
            ax.get_yaxis().set_visible(False)

    # show the colorbar
    if show_colorbar:
        if tile_labels:
            f.subplots_adjust(left=None, bottom=None, right=0.8)
        else:
            f.subplots_adjust(right=0.8, left=None, bottom=None,
                              top=None, wspace=0, hspace=0)
            
        #[left, bottom, width, height]
        h=.6;w=.025
        cbar_ax = f.add_axes([0.85, (1-h)/2, w, h])
        cbar = f.colorbar(im, cax=cbar_ax)#, format='%.0e')        
        if show_cbar_label:
            cbar.set_label(cbar_label)

    return f
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


def plot_tiles_proj(lons, lats, data,  **kwargs):
    
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    # this routine plots the 13 llc faces in Robinson layout
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

    # default projection type
    projection_type = 'robin'
    
    # by default the left most longitude in the global map is -180E.
    user_lon_0 = 0

    # by default use jet colormap
    user_cmap = 'jet'
    
    # by default do not show a colorbar 
    show_colorbar = False

    # by default the colorbar has no label
    show_cbar_label = False

    # by default take the min and max of the values
    cmin = np.nanmin(data)
    cmax = np.nanmax(data)

    # 1/4 degree is the default resolution in lon and lat to interpolate
    dx = .25
    dy = .25
    
    # by default the plot_type is pcolor.
    plot_type = 'pcolor'
    
    # the default number of levels for contourf, 
    num_levels = 20
    
    # default background is to fill continents with gray color
    background_type = 'fc'
       
    # default bounding lat for polar stereographic projection is 50 N
    bound_lat = 50
    
    #%%
    for key in kwargs:
        if key == "lon_0" :
            user_lon_0 = kwargs[key]
        elif key == "cbar":
            show_colorbar = kwargs[key]
        elif key == "user_cmap":
            user_cmap = kwargs[key]
        elif key == "cbar_label":
            cbar_label = kwargs[key]
            show_cbar_label = True
        elif key == "cmin":
            cmin = kwargs[key]
        elif key == "cmax":
            cmax =  kwargs[key]
        elif key == "dx":
            dx =  kwargs[key]
        elif key == "dy":
            dy =  kwargs[key]
        elif key == "plot_type":
            plot_type =  kwargs[key]
        elif key == "num_levels":
            num_levels =  kwargs[key]
        elif key == "projection_type":
            projection_type = kwargs[key]
        elif key == "background_type":
            background_type = kwargs[key]
        elif key == "bounding_lat":
            bound_lat = kwargs[key]
        else:
            print "unrecognized argument ", key     

    #%%
    if type(lons) == xr.core.dataarray.DataArray:
        lons_1d = lons.values.reshape(np.product(lons.values.shape))
        lats_1d = lats.values.reshape(np.product(lats.values.shape))   

    elif type(lons) == np.ndarray:
        lons_1d = lons.reshape(np.product(lons.shape))
        lats_1d = lats.reshape(np.product(lats.shape) )       
    else:
        print 'lons and lats variable either a DataArray or numpy.ndarray'
        print 'lons found type ', type(lons)
        print 'lats found type ', type(lats)        
        return
    
    if type(data) == xr.core.dataarray.DataArray:
        data = data.values


    elif type(data) != np.ndarray:
        print 'data must be either a DataArray or ndarray type \n'
        print 'found type ', type(data)
        return

    #%%
    # To avoid plotting problems around the date line, lon=180E, -180W 
    # I take the approach of plotting the field in two parts, A and B.  
    # Typically part 'A' spans from starting longitude to 180E while part 'B' 
    # spans the from 180E to 360E + starting longitude.  If the starting 
    # longitudes or 0 or 180 special case.
    if user_lon_0 > -180 and user_lon_0 < 180:
        A_left_limit = user_lon_0
        A_right_limit = 180
        B_left_limit =  180
        B_right_limit = 360+user_lon_0
        center_lon = A_left_limit + 180
        
    elif user_lon_0 == 180 or user_lon_0 == -180:
        A_left_limit = -180
        A_right_limit = 0
        B_left_limit =  0
        B_right_limit = 180
        center_lon = 0
    else:
        print 'invalid starting longitude'
        #return

    # the number of degrees spanned in part A and part B
    num_deg_A =  (A_right_limit - A_left_limit)/dx
    num_deg_B =  (B_right_limit - B_left_limit)/dx

    # We will interpolate the data to the new grid.  Store the longitudes to
    # interpolate to for part A and part B
    lon_tmp_d = dict()
    if num_deg_A > 0:
        lon_tmp_d['A'] = np.linspace(A_left_limit, A_right_limit, num_deg_A)
        
    if num_deg_B > 0:
       lon_tmp_d['B'] = np.linspace(B_left_limit, B_right_limit, num_deg_B)

    
    # create the basemap object, 'map'
    if projection_type == 'cyl':
        map = Basemap(projection='cyl',llcrnrlat=-90,urcrnrlat=90,\
                llcrnrlon=A_left_limit, urcrnrlon=B_right_limit, 
                resolution='c')
    
    elif projection_type == 'robin':    
        map = Basemap(projection='robin',lon_0=center_lon, resolution='c')

    elif projection_type == 'stereo':    
        if bound_lat > 0:
            map = Basemap(projection='npstere', boundinglat = bound_lat,
                          lon_0=user_lon_0, resolution='c')
        else:
            map = Basemap(projection='spstere', boundinglat = bound_lat,
                          lon_0=user_lon_0, resolution='c')
    else:
        raise ValueError('projection type must be either "cyl", "robin", or "stereo"')
        print 'found ', projection_type
    
    #%%
    # get a reference to the current figure (or make a figure if none exists)
    if background_type == 'bm':
        map.bluemarble()
        print 'blue marble'
    elif background_type == 'sr':
        map.shadedrelief()
        print 'shaded relief'        
    elif background_type == 'fc':
        map.fillcontinents(color='lightgray',lake_color='lightgray')  

    # prepare for the interpolation or nearest neighbor mapping
    
    # first define the lat lon points of the original data
    orig_grid = pr.geometry.SwathDefinition(lons=lons_1d, lats=lats_1d)
    
    # the latitudes to which we will we interpolate
    lat_tmp = np.linspace(-89.5, 89.5, 90/dy)
    
    # loop through both parts (if they exist), do interpolation and plot
    for key, lon_tmp in lon_tmp_d.iteritems():

        #%%
        new_grid_lon, new_grid_lat = np.meshgrid(lon_tmp, lat_tmp)
    
        
        # define the lat lon points of the two parts. 
        new_grid  = pr.geometry.GridDefinition(lons=new_grid_lon, 
                                               lats=new_grid_lat)
        
        x,y = map(new_grid_lon, new_grid_lat) 
    
        data_latlon_projection = \
            pr.kd_tree.resample_nearest(orig_grid, data, new_grid, 
                                        radius_of_influence=100000, 
                                        fill_value=None) 

        if plot_type == 'pcolor':
            # plot using pcolor 
            im=map.pcolor(x,y, data_latlon_projection, 
                          vmin=cmin, vmax=cmax, cmap=user_cmap)

        elif plot_type == 'contourf':
            # create a set of contours spanning from cmin to cmax over
            # num_levels intervals
            contour_levels = np.linspace(cmin, cmax, num_levels)
            
            # plot using contourf
            im=map.contourf(x,y, data_latlon_projection, num_levels,
                         vmin=cmin, vmax=cmax, cmap=user_cmap, 
                         levels=contour_levels, extend="both")
        else:
            print 'plot type must be either "pcolor" or "contourf"  '
            print 'found type ', plot_type
            #return
        
           
    # draw coastlines, country boundaries, fill continents.
    map.drawcoastlines(linewidth=1)
    # don't plot lat/lon labels for robinson     projection.
    if projection_type == 'robin':      
        map.drawmeridians(np.arange(0,360,30))
        map.drawparallels(np.arange(-90,90,30))
    elif projection_type == 'stereo':      
        map.drawmeridians(np.arange(0,360,30))
        map.drawparallels(np.arange(-90,90,10)) 
    elif projection_type == 'cyl':
        # labels = [left,right,top,bottom]
        map.drawparallels(np.arange(-90,90,30), labels=[True,False,False,False])    
        map.drawmeridians(np.arange(0,360,60),  labels= [False,False, False,True])
    
    #%%
    ax= plt.gca()

    if show_colorbar:
        f=plt.gcf()
        f.subplots_adjust(right=0.8)
        #[left, bottom, width, height]
        h=.6;w=.025
        cbar_ax = f.add_axes([0.85, (1-h)/2, w, h])
        cbar = f.colorbar(im, extend='both', cax=cbar_ax)#, format='%.0e')          

        if show_cbar_label:
            cbar.set_label(cbar_label)
    # set the current axes to be the map, not the colorbar
    plt.sca(ax)

    # return a reference to the figure and the map axes
    return f, ax, im
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




    
    
def unique_color(n):
    
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    # returns one of 13 unique colors.
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if n == 1:
        c='xkcd:red'
    elif n== 2:
        c='xkcd:green'
    elif n== 3:
        c='xkcd:yellow'
    elif n== 4:
        c='xkcd:blue'
    elif n== 5:
        c='xkcd:orange'
    elif n== 6:
        c='xkcd:purple'
    elif n== 7:
        c='xkcd:cyan'
    elif n== 8:
        c='xkcd:magenta'
    elif n== 9:
        c='xkcd:lime green'
    elif n== 10:
        c='xkcd:candy pink'
    elif n== 11:
        c='xkcd:teal'
    elif n== 12:
        c='xkcd:lavender'
    elif n== 13:
        c='xkcd:brown'
    else:
        c='xkcd:mint'

    return c
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon Jul  3 16:11:15 2017

@author: ifenty
"""
from __future__ import division
import numpy as np
import xarray as xr
from copy import deepcopy

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        

def reorient_13_tile_GRID_Dataset_to_latlon_layout(gds, **kwargs):
    
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    #
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    # By default we will keep the Arctic tile aligned with Tile 6.
    aca = 6

    # However, an optional argument allows the alignment to be changed.
    for key in kwargs:
        if key == "Arctic_Align":
            aca = kwargs[key]
    
    #print 'ACA: ', aca
    
    # ROTATION PART 1: GRID PARAMETERS ON THE 'C' AND 'G' POINTS
    # -- step 1: select the grid parameters that are situated on the 
    #            'C' or 'G' points.  These 'rotate' easily because these 
    #            variables have no 'direction' (XC after rotation is still XC)
    #            (except maybe the Angles but haven't sorted those out yet)
    ds_CG = gds[['XC','YC', 'Depth','hFacC', 'RAC', 
                 'AngleCS', 'AngleSN','XG','YG','RAZ', 'land_c']]

    # -- step 2: rotate the new subsetted Dataset 

    ds_CG_r = reorient_13_tile_Dataset_to_latlon_layout_CG_points(ds_CG, **kwargs)

    for key, value in ds_CG_r.variables.iteritems():
        if key not in ds_CG_r.coords:
            ds_CG_r.variables[key].attrs['Arctic_Align'] = aca

    # create two Datasets with subsets of grid parameters situated on 
    # 'U' and 'V' points, respectively.  Rotation of 'U' and 'V' point 
    # variables to the lat-lon orientation requires that the order of the 
    # variables in the two Datasets are 
    # 
    # correct order (e.g., after rotation DXC on tile 6 corresponds with DYC 
    # in tile 8, DYG on tile 6 corresponds with DXG on tile 8 after rotation)
    
    ds_U = gds[['DXC','DYG','hFacW','land_u']]
    ds_V = gds[['DYC','DXG','hFacS','land_v']]    
    
    ds_U_r, ds_V_r = \
        reorient_13_tile_Dataset_to_latlon_layout_UV_points(ds_U, ds_V, **kwargs)

    # merge the rotated Datasets on the C, G, U and V points.
    gds_r = xr.merge([ds_CG_r, ds_U_r, ds_V_r])
    
    for key, value in gds_r.variables.iteritems():
        if key not in gds_r.coords:
            gds_r.variables[key].attrs['Arctic_Align'] = aca
            gds_r.variables[key].attrs['rotated_to_latlon'] = True

    #%%
    gds_r.attrs = gds.attrs
    return gds_r
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




def reorient_13_tile_Dataset_to_latlon_layout_CG_points(ds, **kwargs):
    
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    # this routine takes a Dataset with multiple llc tiles containing
    # variables on the 'C' or 'G' points of the Arakawa-C grid and, if 
    # present, rotates tiles 8-13 so that they line up with tiles 1-6 
    # in a quasi lat-lon layout.  
    #
    # note: the Dataset can be a subset of the 13 llc tiles.
    # 
    # The Actic Cap Tile 7 can be rotated in one of four ways to align it 
    # with tiles 3, 6, 8, or 11.  The default orientation of tile 7 has it 
    # aligned with tile 6 in the 'y' direction and tile 8 in the 'x' direction
    # Tile 7's alignment in the 'y' direction can be changed to a different 
    # tile using the 'Arctic_Align' argument.  
    #
    # For example, to make tile 7 line up with tile 3 in the 'y' direction use
    # the argument Arctic_Align = 3
    #
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    print "\n>>> REORIENTING DATASET TO LATLON LAYOUT, C OR G POINTS"
    # create an empty array that will contain the tiles after rotation
    ds_all = []
    # save the metadata from the Dataset, we'll use it later.
    attrs = ds.attrs
    
    # By default, the Arctic cap will remain aligned with tile 6 in 
    # the 'y' direction
    aca = 6
    
    #%%
    # To make the Arctic cap align with a different tile in the 'y' direction
    # specify the option 'Artic_Cap_Alignment' argument
    for key in kwargs:
        if key == "Arctic_Align":
            aca = kwargs[key]


    #%%            
    # counter for the number of tiles that we've processed
    num_tiles = 0
    
    # loop through all 13 tiles
    for cur_tile in range(1,14):

        # Proceed if cur_tile is in the Dataset
        if cur_tile in ds.tile.values:
            # increment the number of tiles found.
            num_tiles = num_tiles + 1

            # extract the Dataset corresponding with this tile
            cur_ds = ds.sel(tile=cur_tile)
        
            # For tiles 1-6 we do nothing, they are already in a quasi lat-lon 
            # orientation
            
            # Tile 7, the Arctic Cap, can be rotated to align with tile
            # 3, 6, 8, or 11 in the 'y' direction.
            # If the Arctic is be aligned with a different tile it must be rotated
            # by 90 degrees clockwise more than one time.
            if cur_tile == 7:
                if aca == 3:
                    cur_ds = rotate_single_tile_Dataset_CG_points(cur_ds, 
                                                                  rot_k =3)
                elif aca == 6:
                    print 'keeping Arctic cap alignment with tile 6'
                elif aca == 8:                
                    cur_ds = rotate_single_tile_Dataset_CG_points(cur_ds)                
                elif aca == 11:
                    cur_ds = rotate_single_tile_Dataset_CG_points(cur_ds, 
                                                                  rot_k=2)
                else:
                    print 'invalid Arctic cap alignment, leaving unchanged'
    
            # Tiles 8-13 are rotated once clockwise
            elif cur_tile > 7:
                cur_ds = rotate_single_tile_Dataset_CG_points(cur_ds)                
    
            # Append the new, possibly rotated Dataset, to the new Dataset array
            if num_tiles == 1:
                ds_all = cur_ds
            else:
                ds_all = xr.concat([ds_all, cur_ds],'tile')

        else:
            print 'Tile not present in Dataset ', cur_tile
            
    # Give the new Dataset the same metadata as the original Dataset.
    ds_all.attrs = attrs

    for key, value in ds_all.variables.iteritems():
        if key not in ds_all.coords:
            ds_all.variables[key].attrs['Arctic_Align'] = aca 
            ds_all.variables[key].attrs['rotated_to_latlon'] = True
            
    #%%
    return ds_all


#%%
def rotate_single_tile_Dataset_CG_points(ds, **kwargs):
    
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    # Rotates a set of variables in a single Dataset tile
    # by some multiple of 90 degrees in the horizontal plane.  
    #
    # The number of 90 degree rotations is specified by the optional 
    # 'rot_k' argument
    #
    # The variables in this Dataset must all be on the 
    # 'C' or 'G' points of the Arakawa-C grid.  
    #
    # Horizontal dimensions must be the same in both x and y directions.
    #
    # Note: rotation of variables on the 'U' or 'V' points of the Arakawa-C 
    # grid require special attention and use a different subroutine.
    # 
    # One use of this routine is to rotate tiles 8-13 from the original
    # llc tile layout to match the quasi lat-lon layout of tiles 1-6.
    #
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    # First do a little sanity check - if the Dataset has more than one tile 
    # the routine will blindly rotate them all by 90 degrees.  This is dangerous
    # since normally we only want to rotate tiles 7-13 and even then by 
    # different multiples of 90 degrees.
    if 'tile' in ds.coords:
        if ds.tile.size > 1:
            print 'WARNING: you are passing a Dataset with more than one tile to "rotate_single_tile_Dataset_CG_points"'
            
        
    # loop through each variable in the Dataset and select out for rotation 
    # only those variables that are not coordinates
    for key, value in ds.variables.iteritems():
        if key not in ds.coords:
            
            # pull out the variable from the Dataset.  A single variable from
            # the Dataset is a DataArray
            da = ds[key]
            
            # Send the DataArray off to be rotated
            ds[key].values = \
                rotate_single_tile_DataArray_CG_points(da, **kwargs)

            # Assign this new attribute to the variable
            ds[key].attrs['rotated_to_latlon'] = True                        
    #%%                         
    return ds

    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


def rotate_single_tile_DataArray_CG_points(da, **kwargs):
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    # Rotates a variable from a single DataArray tile
    # by some multiple of 90 degrees in the horizontal plane.  
    #
    # The number of 90 degree rotations is specified by the optional 
    # 'rot_k' argument
    #
    # The variable in this DataArray must be on the 
    # 'C' or 'G' points of the Arakawa-C grid.  
    #
    # Note: rotation of variables on the 'U' or 'V' points of the Arakawa-C 
    # grid require special attention and use a different subroutine.
    # 
    # One use of this routine is to rotate tiles 8-13 from the original
    # llc tile layout to match the quasi lat-lon layout of tiles 1-6.
    #
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    # First a sanity check to see if the Dataset has more than one tile.
    if 'tile' in da.coords:
        if da.tile.size > 1:
            print 'WARNING: you are passing a DataArray with more than one tile to "rotate_single_tile_DataArray_CG_points"'

    # by default the array will be rotated clockwise 90 degrees in its 
    # horizontal dimensions rot_k = 1 times.  
    rot_k = 1
    
    # the number of 90 degree rotations can be specified with the optional 
    # 'rot_k' argument
    for key in kwargs:
        if key == "rot_k":
            rot_k = kwargs[key]
    
    #%%
    # get the dimensions of the variable
    num_dims = da.ndim

    # the rotation command depends on how many dimensions the variable
    # has.  The horizontal dimensions are always the last two axes.            
    if num_dims == 2:
        # assuming that the two dimensions are y and x                    
        da.values = np.rot90(da.values, k=rot_k)
        # give this variable a new attribute
        da.attrs['rotated_to_latlon'] = True                        
    elif num_dims == 3:
        # assuming that the last two dimensions are y and x
        da.values = np.rot90(da.values,  k=rot_k , axes=(1,2))
        # give this variable a new attribute
        da.attrs['rotated_to_latlon'] = True                        
 
    elif num_dims == 4:
        # assuming that the last two dimensions are y and x                    
        da.values = np.rot90(da.values,  k=rot_k , axes=(2,3))
        # give this variable a new attribute
        da.attrs['rotated_to_latlon'] = True                        
    else:
        print 'variables must have 2, 3, or 4 dimensions'
        print da.name + ' has ' + str(num_dims) + ' dimensions\n'
        print 'so I am returning ' + da.name + ' unchanged.'
        
    #%%            
    return da.values
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        



def reorient_13_tile_Dataset_to_latlon_layout_UV_points(ds_U, ds_V, **kwargs):
    
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    # Take a pair of Datasets containing variables 
    # on the 'U' or 'V' points of the Arakawa-C grid and rotate 
    # those variables in tiles 8-13 so that they line up with tiles 1-6 
    # in a quasi lat-lon layout.  
    #
    # The Arctic Cap (tile 7) can be rotated so 
    # that it aligns with tiles 3, 6, 8, or 11 in the 'y' direction.
    # The default orientation of tile 7 has it 
    # aligned with tile 6 in the 'y' direction and tile 8 in the 'x' direction.
    #
    # To change the alignment of tile 7 in the 'y' direction to a different
    # tile specify the target tile with the 'Arctic_Align' argument.  
    #
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    # create an empty array that will contain the tiles after rotation
    ds_U_all = []
    ds_V_all = []    

    # save the metadata from the Dataset, we'll use it later.
    U_attrs = ds_U.attrs
    V_attrs = ds_V.attrs 
    
    # By default, the Arctic cap will remain aligned with tile 6 in 
    # the 'y' direction
    aca = 6
    
    #%%
    # To make the Arctic cap align with a different tile in the 'y' direction
    # specify the option 'Artic_Cap_Alignment' argument
    for key in kwargs:
        if key == "Arctic_Align":
            aca = kwargs[key]

    #print "Reorient UV ACA: ", aca
    #%%            
    # counter for the number of tiles that we've processed
    num_tiles = 0
    
    # loop through all 13 tiles
    for cur_tile in range(1,14):

        #print cur_tile

        # Check to make sure cur_tile is in the ds_U Dataset (if it is assume
        # is it also in ds_V)
        if cur_tile in ds_U.tile.values:
            #print 'TILE: ', cur_tile
            # increment the number of tiles found.
            num_tiles = num_tiles + 1

            # extract the Dataset corresponding with this tile
            cur_ds_U = ds_U.sel(tile=cur_tile)
            cur_ds_V = ds_V.sel(tile=cur_tile)
            
            # For tiles 1-6 we do nothing, they are already in a quasi lat-lon 
            # orientation
            
            # Tile 7, the Arctic Cap, can be rotated to align with tile
            # 3, 6, 8, or 11 in the 'y' direction.
            # If the Arctic is be aligned with a different tile it must be 
            # rotated by 90 degrees clockwise more than one time.
            if cur_tile == 7:
                if aca == 3:
                    cur_ds_U, cur_ds_V = \
                        rotate_single_tile_Datasets_UV_points(cur_ds_U, 
                                                              cur_ds_V,
                                                              rot_k=3)
                elif aca == 6:
                    print 'keeping Arctic cap alignment with tile 6'
                elif aca == 8:                
                    cur_ds_U, cur_ds_V = \
                        rotate_single_tile_Datasets_UV_points(cur_ds_U, 
                                                              cur_ds_V)
                elif aca == 11:
                    cur_ds_U, cur_ds_V = \
                        rotate_single_tile_Datasets_UV_points(cur_ds_U, 
                                                              cur_ds_V,
                                                              rot_k=2)
                else:
                    print 'invalid Arctic cap alignment, leaving unchanged'
    
            # Tiles 8-13 are rotated once clockwise
            elif cur_tile > 7:
                cur_ds_U, cur_ds_V = \
                    rotate_single_tile_Datasets_UV_points(cur_ds_U, 
                                                          cur_ds_V,)                
    
            # Append the new Dataset to the new Dataset array
            if num_tiles == 1:
                ds_U_all = cur_ds_U
                ds_V_all = cur_ds_V
            else:
                ds_U_all = xr.concat([ds_U_all, cur_ds_U],'tile')
                ds_V_all = xr.concat([ds_V_all, cur_ds_V],'tile')            
        else:
            print "Tile not included in Dataset ", cur_tile
            
    # Give the new Dataset the same metadata as the original Dataset.
    ds_U_all.attrs = U_attrs
    ds_V_all.attrs = V_attrs 
    #%%
    
    for key, value in ds_U_all.variables.iteritems():
        if key not in ds_U_all.coords:
            ds_U_all.variables[key].attrs['Arctic_Align'] = aca
            ds_U_all.variables[key].attrs['rotated_to_latlon'] = True

    for key, value in ds_V_all.variables.iteritems():
        if key not in ds_V_all.coords:
            ds_V_all.variables[key].attrs['Arctic_Align'] = aca
            ds_V_all.variables[key].attrs['rotated_to_latlon'] = True            
            
    return ds_U_all, ds_V_all



#%%
def rotate_single_tile_Datasets_UV_points(ds_U, ds_V, **kwargs):
    
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    # Rotates a set of variables in two Datasets for a single tile 
    # by some multiple of 90 degrees in the horizontal plane.  
    #
    # The number of 90 degree rotations is specified by the optional 
    # 'rot_k' argument
    #
    # The variables in Dataset ds_U must all be on the 'U' points and 
    # the variables in Dataset ds_V must all be on the 'V' points 
    # of the Arakawa-C grid.  
    #
    # One use of this routine is to rotate tiles 8-13 from the original
    # llc tile layout to match the quasi lat-lon layout of tiles 1-6.
    #
    # Corresponding U and V point variables must be provided in the identical
    # order in the Datasets. e.g., DXC corresponds with DYC because the 
    # the direction of X and Y change after an odd number of rotations.
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    # First a sanity check - if the Dataset has more than one tile 
    # the routine will blindly rotate them all by 90 degrees. This is dangerous
    # since normally we only want to rotate tiles 7-13 and even then by 
    # different multiples of 90 degrees.
    if 'tile' in ds_U.coords:
        if ds_U.tile.size > 1:
            print 'WARNING: you are passing a Dataset with more than one tile'
            
        
    # loop through each variable in the Dataset and select out for rotation 
    # only those variables that are not coordinates
    
    print '\nROTATING TILE ', ds_U.tile.values
    
    num_vars = 0
    ki = 0
    for key, value in ds_U.variables.iteritems():
        ki = ki+1
        if key not in ds_U.coords:
            
            num_vars = num_vars+1
            
            cur_V_key = ds_V.variables.keys()[ki-1]
            cur_U_key = ds_U.variables.keys()[ki-1]

            print 'PAIRING ' + cur_U_key + ' with ' + cur_V_key
            
            da_U = ds_U[cur_U_key]
            da_V = ds_V[cur_V_key]
 
            # Send the DataArray off to be rotated
            da_Ur, da_Vr = \
                rotate_single_tile_DataArrays_UV_points(da_U, da_V, **kwargs)

            ds_U[cur_U_key].values = da_Ur.values
            ds_V[cur_V_key].values = da_Vr.values
            
            ds_U[cur_U_key].attrs = da_Ur.attrs
            ds_V[cur_V_key].attrs = da_Vr.attrs
            
            
    #%%                         
    return ds_U, ds_V

    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

def rotate_single_tile_DataArrays_UV_points(da_U, da_V, **kwargs):
    
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    # Rotate a pair of DataArray variables for a single tile by some
    # multiple of 90 degrees.  
    # da_U is a variable on the 'U' points of 
    # the tile while da_V is the variable on the 'V' points of the tile.  
    #
    # Note, DataArrays are essentially single variable versions of Datasets.
    #   da_U = ds['U point variable name']
    #   da_V = ds['V point variable name']    
    #
    # The DataArrays can have 2, 3, or 4 dimensions.
    #
    # If the number of 90 degree rotations is odd, the variable that was 
    # originally on the 'U' points will be situated on the 'V' points and 
    # vice versa because the meaning of x and y will be swapped.  The 
    # rotated version of U will come from V and vice versa.  
    # On the other hand, if the 
    # number of 90 degree rotations is even the rotated version of U comes 
    # from U and vice versa.
    #
    # If the variables involve velocity or fluxes, the variables may need
    # to change sign after rotation. 
    # For example, consider rotation of velocity variables by 90 degrees 
    # in tiles 8-13 in the original llc tile layout.
    # After rotation, 
    # +U velocities in the +x direction of tiles 8-13 become -V velocties 
    # in the y direction after rotation (sign change) 
    # +V velocities in the +y direction of tiles 8-13 become +U velocities
    # in the x direction after rotation (no sign change)
    #
    # If you want the signs to be flipped in either U or V, specify with
    # the u_sign_flip or v_sign_flip parameters.  To flip the
    # sign of 'U' variables in tile 8 after rotating them by 90 degrees:
    # >da_U = ds['UVEL'].sel(tile=8)
    # >da_V = ds['VVEL'].sel(tile=8)    
    # >da_Ur, da_Vr = 
    #     rotate_single_tile_DataArrays_UV_points(da_U, da_V, 
    #                                             u_sign_flip=True):
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    # by default the array will be rotated clockwise 90 degrees in its 
    # horizontal dimensions rot_k = 1 times.  
    rot_k = 1
    
    #%%
    # the number of 90 degree rotations can be specified with the optional 
    # 'rot_k' argument
    for key in kwargs:
        if key == "rot_k":
            rot_k = kwargs[key]

    #%%
    # By default we do not flip the sign of U and V
    flip_U = False
    flip_V = False
    #%%
    # look for the u_sign_flip and the v_sign_flip arguments
    for key in kwargs:
        if key == "u_sign_flip" and kwargs[key] == True:
            flip_U = True

        elif key == "v_sign_flip" and kwargs[key] == True:
            flip_V = True

    #%%
    if flip_U:
        print "Flipping the sign of the variable on the 'U' point \n"
        da_U = da_U * -1
    if flip_V:
        print "Flipping the sign of the variable on the 'V' point \n"
        da_V = da_V * -1

    da_U_new = deepcopy(da_U)
    da_V_new = deepcopy(da_V)  
       
    # in the special case where the number of rotations is even 
    # then the array dimensions do not change and the rotated version of
    # da_U comes from da_U and the rotated version of da_V comes from da_V
    if np.mod(rot_k, 2) == 0:
        print "Even number of rotations\n"
        
        da_U_new[:] = np.rot90(da_U, k=rot_k, axes=(-2,-1))
        da_V_new[:] = np.rot90(da_V, k=rot_k, axes=(-2,-1))
        
    else:
        rot_U = np.rot90(deepcopy(da_U), k=rot_k, axes=(-2,-1))
        rot_V = np.rot90(deepcopy(da_V), k=rot_k, axes=(-2,-1))
        
        da_U_new[:] = rot_V
        da_V_new[:] = rot_U
      
    da_U_new.attrs['rotated_to_latlon'] = True
    da_V_new.attrs['rotated_to_latlon'] = True   
        
    print da_U_new.attrs
    print da_V_new.attrs
    
    #%%   
    return da_U_new, da_V_new
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
           



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Thu Feb  8 11:01:48 2018

@author: ifenty
"""
from __future__ import division
import numpy as np
import matplotlib.pylab as plt
import xarray as xr

# import all function from the 'mpl_toolkits.basemap' module available with the 
# prefix 'Basemap'
from mpl_toolkits.basemap import Basemap


def plot_latlon_interp_proj(lons, lats, data,  **kwargs):
    
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    # this routine plots the 0.5 degree interpolated fields
    # in either cylindrical or robinson projections
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    if type(lons) == xr.DataArray:
        lons = lons.values
    elif type(lons) != np.ndarray:
        print 'lons must be either a DataArray or ndarray type \n'
        print 'found type ', type(lons)

    if type(lats) == xr.DataArray:
        lats = lats.values
    elif type(lats) != np.ndarray:
        print 'lats must be either a DataArray or ndarray type \n'
        print 'found type ', type(lats)
        
    if type(data) == xr.DataArray:
        data = data.values
    elif type(data) != np.ndarray:
        print 'data must be either a DataArray or ndarray type \n'
        print 'found type ', type(data)

    # default projection type
    projection_type = 'robin'
    
    # by default the left most longitude in the global map is -180E.
    user_lon_0 = 0

    # by default use jet colormap
    user_cmap = 'jet'
    
    # by default do not show a colorbar 
    show_colorbar = False

    # by default the colorbar has no label
    show_cbar_label = False

    # by default take the min and max of the values
    cmin = np.nanmin(data)
    cmax = np.nanmax(data)
   
    # by default the plot_type is pcolor.
    plot_type = 'pcolor'
    
    # the default number of levels for contourf, 
    num_levels = 20
    
    # default background is to fill continents with gray color
    background_type = 'fc'
        
    #%%
    for key in kwargs:
        if key == "lon_0" :
            user_lon_0 = kwargs[key]
        elif key == "cbar":
            show_colorbar = kwargs[key]
        elif key == "user_cmap":
            user_cmap = kwargs[key]
        elif key == "cbar_label":
            cbar_label = kwargs[key]
            show_cbar_label = True
        elif key == "cmin":
            cmin = kwargs[key]
        elif key == "cmax":
            cmax =  kwargs[key]
        elif key == "plot_type":
            plot_type =  kwargs[key]
        elif key == "num_levels":
            num_levels =  kwargs[key]
        elif key == "projection_type":
            projection_type = kwargs[key]
        elif key == "background_type":
            background_type = kwargs[key]
        else:
            print "unrecognized argument ", key     

    #%%
    # To avoid plotting problems around the date line, lon=180E, -180W 
    # I take the approach of plotting the field in two parts, A and B.  
    # Typically part 'A' spans from starting longitude to 180E while part 'B' 
    # spans the from 180E to 360E + starting longitude.  If the starting 
    # longitudes or 0 or 180 special case.
    
    two_parts = False
    if user_lon_0 == 180 or user_lon_0 == -180:
        A_left_limit = -180
        A_right_limit = 0
        B_left_limit =  0
        B_right_limit = 180
        center_lon = 0

    elif user_lon_0 > -180 and user_lon_0 < 180:
        A_left_limit = user_lon_0
        A_right_limit = 180
        B_left_limit =  180
        B_right_limit = 360+user_lon_0
        center_lon = A_left_limit + 180
        two_parts = True

        tmp = lons[0,:]
        idx_AL = np.searchsorted(tmp,A_left_limit, side="left")
        idx_AR = np.searchsorted(tmp,A_right_limit, side="left")
    
        tmpB = tmp+360;
        idx_BL = np.searchsorted(tmpB,B_left_limit, side="left")
        idx_BR = np.searchsorted(tmpB,B_right_limit, side="left")

        print idx_AL, idx_AR, idx_BL, idx_BR
    else:
        print 'invalid starting longitude'
        #return

    
    #%%
    # create the basemap object, 'map'
    if projection_type == 'cyl':
        map = Basemap(projection='cyl',llcrnrlat=-90,urcrnrlat=90,\
                llcrnrlon=A_left_limit, urcrnrlon=B_right_limit, 
                resolution='c')
    
    elif projection_type == 'robin':    
        map = Basemap(projection='robin',lon_0=center_lon, resolution='c')

    else:
        print 'projection type must be either "cyl" or "robin" '  
        print 'found ', projection_type
        #return
    
    #%%
    # get a reference to the current figure (or make a figure if none exists)
    f = plt.gcf()

    if background_type == 'bm':
        map.bluemarble()
        print 'blue marble'
    elif background_type == 'sr':
        map.shadedrelief()
        print 'shaded relief'        
    elif background_type == 'fc':
        map.fillcontinents(color='lightgray',lake_color='lightgray')  
        print 'gray background'                

    x,y = map(lons, lats) 
    xb,yb = map(lons+360, lats) 
    
    #%%
    if plot_type == 'pcolor':
        # plot using pcolor 
        if two_parts:
            im=map.pcolor(x[:,idx_AL:idx_AR],y[:,idx_AL:idx_AR], 
                          data[:,idx_AL:idx_AR], 
                          vmin=cmin, vmax=cmax, cmap=user_cmap)
            im=map.pcolor(xb[:,idx_BL:idx_BR],y[:,idx_BL:idx_BR], 
                          data[:,idx_BL:idx_BR], 
                          vmin=cmin, vmax=cmax, cmap=user_cmap)
        else:
            im=map.pcolor(x,y,data,
                          vmin=cmin, vmax=cmax, cmap=user_cmap)

    elif plot_type == 'contourf':
        # create a set of contours spanning from cmin to cmax over
        # num_levels intervals
        contour_levels = np.linspace(cmin, cmax, num_levels)
        
        # plot using contourf
        if two_parts:
            im=map.contourf(x[:,idx_AL:idx_AR],y[:,idx_AL:idx_AR], 
                            data[:,idx_AL:idx_AR], 
                            num_levels,
                            vmin=cmin, vmax=cmax, cmap=user_cmap, 
                            levels=contour_levels, extend="both")
            im=map.contourf(xb[:,idx_BL:idx_BR],y[:,idx_BL:idx_BR], 
                            data[:,idx_BL:idx_BR], 
                            num_levels,
                            vmin=cmin, vmax=cmax, cmap=user_cmap, 
                            levels=contour_levels, extend="both")
        else:
            im=map.contourf(x,y,data,
                            num_levels,
                            vmin=cmin, vmax=cmax, cmap=user_cmap, 
                            levels=contour_levels, extend="both")
    else:
        print 'plot type must be either "pcolor" or "contourf"  '
        print 'found type ', plot_type
        #return
        
    #%%
           
    # draw coastlines, country boundaries, fill continents.
    map.drawcoastlines(linewidth=1)
    # don't plot lat/lon labels for robinson     projection.
    if projection_type == 'robin':      
        map.drawmeridians(np.arange(0,330,30))
        map.drawparallels(np.arange(-60,61,30))
    else:
        # labels = [left,right,top,bottom]
        map.drawparallels(np.arange(-60,61,30), labels=[True,False,False,False])    
        map.drawmeridians(np.arange(0,301,60),  labels= [False,False, False,True])
    
    #%%
    ax= plt.gca()

    if show_colorbar:
        f=plt.gcf()
        f.subplots_adjust(right=0.8)
        #[left, bottom, width, height]
        h=.6;w=.025
        cbar_ax = f.add_axes([0.85, (1-h)/2, w, h])
        cbar = f.colorbar(im, extend='both', cax=cbar_ax)#, format='%.0e')          

        if show_cbar_label:
            cbar.set_label(cbar_label)
    # set the current axes to be the map, not the colorbar
    plt.sca(ax)

    #%%
    # return a reference to the figure and the map axes
    return f, ax, im
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


def plot_latlon_interp(lons, lats, data,  **kwargs):
    
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    # 
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    if type(lons) == xr.DataArray:
        lons = lons.values
    elif type(lons) != np.ndarray:
        print 'lons must be either a DataArray or ndarray type \n'
        print 'found type ', type(lons)

    if type(lats) == xr.DataArray:
        lats = lats.values
    elif type(lats) != np.ndarray:
        print 'lats must be either a DataArray or ndarray type \n'
        print 'found type ', type(lats)
        
    if type(data) == xr.DataArray:
        data = data.values
    elif type(data) != np.ndarray:
        print 'data must be either a DataArray or ndarray type \n'
        print 'found type ', type(data)


    # by default use jet colormap
    user_cmap = 'jet'
    
    # by default do not show a colorbar 
    show_colorbar = False

    # by default the colorbar has no label
    show_cbar_label = False

    # by default take the min and max of the values
    cmin = np.nanmin(data)
    cmax = np.nanmax(data)
   
    # by default the plot_type is pcolor.
    plot_type = 'pcolor'
    
    # the default number of levels for contourf, 
    num_levels = 20
    
    #%%
    for key in kwargs:
        if key == "cbar":
            show_colorbar = kwargs[key]
        elif key == "user_cmap":
            user_cmap = kwargs[key]
        elif key == "cbar_label":
            cbar_label = kwargs[key]
            show_cbar_label = True
        elif key == "cmin":
            cmin = kwargs[key]
        elif key == "cmax":
            cmax =  kwargs[key]
        elif key == "plot_type":
            plot_type =  kwargs[key]
        elif key == "num_levels":
            num_levels =  kwargs[key]
        else:
            print "unrecognized argument ", key     

    #%%
    # get a reference to the current figure (or make a figure if none exists)
    f = plt.gcf()

    #%%
    if plot_type == 'pcolor':
        # plot using pcolor 
        plt.pcolor(lons,lats,data,
                   vmin=cmin, vmax=cmax, 
                   cmap=user_cmap)

    elif plot_type == 'contourf':
        # create a set of contours spanning from cmin to cmax over
        # num_levels intervals
        contour_levels = np.linspace(cmin, cmax, num_levels)
        
        plt.contourf(lons,lats,data,
                     num_levels, 
                     vmin=cmin, vmax=cmax, cmap=user_cmap, 
                     levels=contour_levels, extend="both")
        
    else:
        print 'plot type must be either "pcolor" or "contourf"  '
        print 'found type ', plot_type
        #return
        
  
    #%%
    ax= plt.gca()

    if show_colorbar:
        f=plt.gcf()
        f.subplots_adjust(right=0.8)
        #[left, bottom, width, height]
        h=.6;w=.025
        cbar_ax = f.add_axes([0.85, (1-h)/2, w, h])
        cbar = f.colorbar(im, extend='both', cax=cbar_ax)#, format='%.0e')          

        if show_cbar_label:
            cbar.set_label(cbar_label)
    # set the current axes to be the map, not the colorbar
    plt.sca(ax)

    #%%
    # return a reference to the figure and the map axes
    return f, ax, im
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



from distutils.core import setup
setup(
  name = 'ecco_v4_py',
  packages = ['ecco_v4_py'], # this must be the same as the name above
  version = '0.143',
  description = 'Estimating the Circulation and Climate of the Ocean (ECCO) Version 4 Python Package',
  author = 'Ian Fenty',
  author_email = 'ifenty@gmail.com',
  url = 'https://github.com/ECCO-GROUP/ECCOv4-py',
  keywords = ['ecco','climate','mitgcm','estimate','circulation','climate'],
  install_requires=[
      'xarray',
      'pyproj',
      'matplotlib'],
  classifiers=[
      'Development Status :: 3 - Alpha',
      'Intended Audience :: Science/Research', 
      'License :: OSI Approved :: MIT License',
      'Natural Language :: English',
      'Programming Language :: Python',
      'Programming Language :: Python :: 2.7',
      'Topic :: Scientific/Engineering :: Physics'
  ]
)
